@@ -1,8 +1,8 @@
 /*
-  $Id: Frog.h 14344 2012-02-27 16:40:05Z sloot $
+  $Id: Frog.h 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/Frog.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
   
   This file is part of frog
@@ -32,36 +32,39 @@
 
 #include <set>
 #include "ticcutils/LogStream.h"
-#include "libfolia/foliautils.h"
-#include "frog/Configuration.h"
+#include "ticcutils/Configuration.h"
+#include "ticcutils/Timer.h"
+#include "libfolia/document.h"
 
 std::string prefix( const std::string&, const std::string& );
 bool existsDir( const std::string& );
 std::string& trim(std::string &str);
+std::string getTime();
 
-void getFileNames( const std::string&, std::set<std::string>& );
+void getFileNames( const std::string&, const std::string&, 
+		   std::set<std::string>& );
 
 extern TiCC::LogStream *theErrLog;
 
 extern int tpDebug;
-extern Configuration configuration;
+extern TiCC::Configuration configuration;
 
 class TimerBlock{
 public:
-  Common::Timer parseTimer;
-  Common::Timer tokTimer;
-  Common::Timer mblemTimer;
-  Common::Timer mbmaTimer;
-  Common::Timer mwuTimer;
-  Common::Timer tagTimer;
-  Common::Timer iobTimer;
-  Common::Timer nerTimer;
-  Common::Timer prepareTimer;
-  Common::Timer pairsTimer;
-  Common::Timer relsTimer;
-  Common::Timer dirTimer;
-  Common::Timer csiTimer;
-  Common::Timer frogTimer;
+  TiCC::Timer parseTimer;
+  TiCC::Timer tokTimer;
+  TiCC::Timer mblemTimer;
+  TiCC::Timer mbmaTimer;
+  TiCC::Timer mwuTimer;
+  TiCC::Timer tagTimer;
+  TiCC::Timer iobTimer;
+  TiCC::Timer nerTimer;
+  TiCC::Timer prepareTimer;
+  TiCC::Timer pairsTimer;
+  TiCC::Timer relsTimer;
+  TiCC::Timer dirTimer;
+  TiCC::Timer csiTimer;
+  TiCC::Timer frogTimer;
 };
 
 #endif
