@@ -1,8 +1,8 @@
 /*
-  $Id: cgn_tagger_mod.h 14489 2012-03-22 11:37:02Z sloot $
+  $Id: cgn_tagger_mod.h 15572 2013-01-07 14:59:11Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/cgn_tagger_mod.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -29,20 +29,21 @@
 #ifndef CGN_TAGGER_MOD_H
 #define CGN_TAGGER_MOD_H
 
-class Configuration;
+#include "mbt/MbtAPI.h"
 
 class CGNTagger {
  public:
   CGNTagger();
   ~CGNTagger();
-  bool init( const Configuration& );
+  bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& ) const;
   void addTag( folia::Word *, const std::string&, double );
-  void Classify( folia::Sentence * );
+  void Classify( const std::vector<folia::Word *>& );
+  std::vector<TagResult> tagLine( const std::string& );
   std::string getTagset() const { return tagset; };
  private:
   MbtAPI *tagger;  
-  TiCC::LogStream *cgnLog;
+  LogStream *cgnLog;
   int debug;
   std::string tagset;
   std::string version;
