@@ -1,8 +1,8 @@
 /*
-  $Id: iob_tagger_mod.h 14478 2012-03-19 16:10:38Z sloot $
+  $Id: iob_tagger_mod.h 15572 2013-01-07 14:59:11Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/iob_tagger_mod.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -29,27 +29,24 @@
 #ifndef IOB_TAGGER_MOD_H
 #define IOB_TAGGER_MOD_H
 
-class Configuration;
-
 class IOBTagger {
  public:
   IOBTagger();
   ~IOBTagger();
-  bool init( const Configuration& );
+  bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& ) const;
-  void Classify( folia::Sentence * );
+  void Classify( const std::vector<folia::Word *>& );
   std::string getTagset() const { return tagset; };
  private:
   void addChunk( folia::ChunkingLayer *, 
 		 const std::vector<folia::Word*>&,
 		 const std::vector<double>&,
 		 const std::string& );
-  void addIOBTags( folia::Sentence *, 
-		   const std::vector<folia::Word*>&,
+  void addIOBTags( const std::vector<folia::Word*>&,
 		   const std::vector<std::string>&,
 		   const std::vector<double>& );
   MbtAPI *tagger;
-  TiCC::LogStream *iobLog;
+  LogStream *iobLog;
   int debug;
   std::string version;
   std::string tagset;
