@@ -1,8 +1,8 @@
 /*
-  $Id: mwu_chunker_mod.h 14478 2012-03-19 16:10:38Z sloot $
+  $Id: mwu_chunker_mod.h 15877 2013-04-02 15:57:51Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mwu_chunker_mod.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
   
   This file is part of frog.
@@ -29,8 +29,6 @@
 #ifndef MWU_CHUNKER_H
 #define MWU_CHUNKER_H
 
-#include "libfolia/folia.h"
-
 class mwuAna {
   friend std::ostream& operator<< (std::ostream&, const mwuAna& );
  public:
@@ -57,17 +55,15 @@ class mwuAna {
 
 #define mymap2 std::multimap<std::string, std::vector<std::string> >
 
-class Configuration;
-
 class Mwu {
   friend std::ostream& operator<< (std::ostream&, const Mwu& );
  public:
   Mwu();
   ~Mwu();
   void reset();
-  bool init( const Configuration& );
+  bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& ) const;
-  void Classify( folia::Sentence * );
+  void Classify( const std::vector<folia::Word *>& );
   void add( folia::Word * );
   std::string getTagset() const { return tagset; };
   std::vector<mwuAna*>& getAna(){ return mWords; };
