@@ -1,8 +1,8 @@
 /*
-  $Id: ner_tagger_mod.h 14447 2012-03-12 16:34:56Z sloot $
+  $Id: ner_tagger_mod.h 15572 2013-01-07 14:59:11Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/ner_tagger_mod.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -29,23 +29,20 @@
 #ifndef NER_TAGGER_MOD_H
 #define NER_TAGGER_MOD_H
 
-class Configuration;
-
 class NERTagger {
  public:
   NERTagger();
   ~NERTagger();
-  bool init( const Configuration& );
-  void Classify( folia::Sentence * );
+  bool init( const TiCC::Configuration& );
+  void Classify( const std::vector<folia::Word *>& );
   void addDeclaration( folia::Document& ) const;
-  void addNERTags( folia::Sentence *, 
-		   const std::vector<folia::Word*>&,
+  void addNERTags( const std::vector<folia::Word*>&,
 		   const std::vector<std::string>&,
 		   const std::vector<double>& );
   std::string getTagset() const { return tagset; };
  private:
   MbtAPI *tagger;
-  TiCC::LogStream *nerLog;
+  LogStream *nerLog;
   int debug;
   std::string version;
   std::string tagset;
