@@ -1,8 +1,8 @@
 /*
-  $Id: ucto_tokenizer_mod.h 14267 2012-02-15 13:05:47Z sloot $
+  $Id: ucto_tokenizer_mod.h 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/ucto_tokenizer_mod.h $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
   
   This file is part of frog.
@@ -29,22 +29,22 @@
 #ifndef UCTO_TOKENIZER_MOD_H
 #define UCTO_TOKENIZER_MOD_H
 
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
 #include "ucto/tokenize.h"
 
-class Configuration;
-
 class UctoTokenizer {
  public:
   UctoTokenizer();
   ~UctoTokenizer() { delete tokenizer; delete uctoLog; };
-  bool init( const Configuration&, const std::string&, bool );
+  bool init( const TiCC::Configuration&, const std::string&, bool );
+  void setUttMarker( const std::string& );
   void setSentencePerLineInput( bool );
-  void setInputEncoding( const std::string & );
+  void setInputEncoding( const std::string& );
+  void setQuoteDetection( bool );
   void setInputXml( bool );
+  void setTextClass( const std::string& );
   folia::Document tokenize( std::istream& );
   bool tokenize( folia::Document& );
+  std::vector<std::string> tokenize( const std::string&  );
  private:
   Tokenizer::TokenizerClass *tokenizer;
   TiCC::LogStream *uctoLog;
