@@ -1,8 +1,8 @@
 /*
-  $Id: Frog-util.cxx 14344 2012-02-27 16:40:05Z sloot $
+  $Id: Frog-util.cxx 15572 2013-01-07 14:59:11Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog-util.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog
@@ -61,7 +61,9 @@ bool existsDir( const string& dirName ){
   return result;
 }
 
-void getFileNames( const string& dirName, set<string>& fileNames ){
+void getFileNames( const string& dirName,
+		   const string& ext, 
+		   set<string>& fileNames ){
   DIR *dir = opendir( dirName.c_str() );
   if ( !dir )
     return;
@@ -69,14 +71,18 @@ void getFileNames( const string& dirName, set<string>& fileNames ){
     struct stat sb;
     struct dirent *entry = readdir( dir );
     while ( entry ){
-      if (entry->d_name[0] != '.') {
-        string fullName = dirName + "/" + entry->d_name;
-        if ( stat( fullName.c_str(), &sb ) >= 0 ){
+      if ( entry->d_name[0] != '.' ) {
+	string filename = entry->d_name;
+	if ( ext.empty() ||
+	     filename.rfind( ext ) != string::npos ) {
+	  string fullName = dirName + "/" + filename;
+	  if ( stat( fullName.c_str(), &sb ) >= 0 ){
             if ( (sb.st_mode & S_IFMT) == S_IFREG )
-                fileNames.insert( entry->d_name );
-        }
-       }
-       entry = readdir( dir );
+	      fileNames.insert( entry->d_name );
+	  }
+	}
+      }
+      entry = readdir( dir );
     }
     closedir( dir );
   }
@@ -101,3 +107,13 @@ string& trim(string &str)  //3rd party function. TODO: test if utf-8 safe? (MvG)
     return str;
 }
 
+string getTime() {
+  time_t Time;
+  time(&Time);
+  tm curtime;
+  localtime_r(&Time,&curtime);
+  char buf[256];
+  strftime( buf, 100, "%Y-%m-%dT%X", &curtime );
+  string res = buf;
+  return res;
+}
