@@ -1,8 +1,8 @@
 /*
-  $Id: Frog.cxx 14574 2012-03-28 13:38:44Z sloot $
+  $Id: Frog.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -49,9 +49,7 @@
 
 #include "config.h"
 #include "frog/Frog.h"
-#include "frog/Configuration.h"
-#include "libfolia/document.h"
-#include "libfolia/folia.h"
+#include "ticcutils/Configuration.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/mbma_mod.h"
 #include "frog/mblem_mod.h"
@@ -63,6 +61,7 @@
 
 using namespace std;
 using namespace folia;
+using namespace TiCC;
 
 LogStream my_default_log( cerr, "frog-", StampMessage ); // fall-back
 LogStream *theErrLog = &my_default_log;  // fill the externals
@@ -72,10 +71,12 @@ string testDirName;
 string tmpDirName;
 string outputFileName;
 string docid = "untitled";
+string textclass;
 bool wantOUT;
 string XMLoutFileName;
 bool doXMLin;
 bool doXMLout;
+bool doKanon;
 string outputDirName;
 string xmlDirName;
 set<string> fileNames;
@@ -83,6 +84,8 @@ string ProgName;
 int tpDebug = 0; //0 for none, more for more output
 unsigned int maxParserTokens = 0; // 0 for unlimited
 bool doTok = true;
+bool doLemma = true;
+bool doMorph = true;
 bool doMwu = true;
 bool doIOB = true;
 bool doNER = true;
@@ -90,8 +93,10 @@ bool doParse = true;
 bool doDirTest = false;
 bool doServer = false;
 bool doSentencePerLine = false;
+bool doQuoteDetection = false;
 string listenport = "void";
 string encoding;
+string uttmark = "";
 
 /* assumptions:
    each components gets its own configfile per cmdline options
@@ -121,14 +126,16 @@ void usage( ) {
   cout << "\t============= INPUT MODE (mandatory, choose one) ========================\n"
        << "\t -e <encoding>          specify encoding of the input (default UTF-8)\n"
        << "\t -t <testfile>          Run frog on this file\n"
-       << "\t -x <testfile>          Run frog on this FoLiA XML file\n"
-       << "\t -S <port>              Run as server instead of reading from testfile\n"
+       << "\t -x <testfile>          Run frog on this FoLiA XML file. Or the files form 'testdir'\n"
+       << "\t --textclass=<cls>      use the specified class to for search text in the the FoLia docs.\n"
        << "\t --testdir=<directory>  All files in this dir will be tested\n"
+       << "\t --uttmarker=<mark>     utterances are separated by 'mark' symbols"
+       << "\t                        (default none)\n"
        << "\t -n                     Assume input file to hold one sentence per line\n"
        << "\t --max-parser-tokens=<n> inhibit parsing when a sentence contains over 'n' tokens. (default: no limit)\n"
-
+       << "\t -Q                   Enable quote detection in tokeniser.\n"
        << "\t============= MODULE SELECTION ==========================================\n"
-       << "\t --skip=[mptnc]    Skip Tokenizer (t), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
+       << "\t --skip=[mptncla]    Skip Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
        << "\t============= CONFIGURATION OPTIONS =====================================\n"
        << "\t -c <filename>    Set configuration file (default " << configFileName << ")\n"
        << "\t============= OUTPUT OPTIONS ============================================\n"      
@@ -143,7 +150,11 @@ void usage( ) {
        << "\t -h. give some help.\n"
        << "\t -V or --version .   Show version info.\n"
        << "\t -d <debug level>    (for more verbosity)\n"
-       << "\t -Q                   Enable quote detection in tokeniser.\n";
+       << "\t -S <port>              Run as server instead of reading from testfile\n"
+#ifdef HAVE_OPENMP      
+       << "\t --threads=<n>       Use a maximun of 'n' threads. Default: all we can get. \n"
+#endif
+       << "\t                     (but always 1 for server mode)\n";
 }
 
 //**** stuff to process commandline options *********************************
@@ -177,7 +188,7 @@ bool parse_args( TimblOpts& Opts ) {
   };
 
   if ( configuration.fill( configFileName ) ){
-    cerr << "config read from: " << configFileName << endl;
+    *Log(theErrLog) << "config read from: " << configFileName << endl;
   }
   else {
     cerr << "failed te read configuration from! '" << configFileName << "'" << endl;
@@ -187,7 +198,7 @@ bool parse_args( TimblOpts& Opts ) {
 
   // debug opts
   if ( Opts.Find ('d', value, mood)) {
-    if ( !Timbl::stringTo<int>( value, tpDebug ) ){
+    if ( !stringTo<int>( value, tpDebug ) ){
       *Log(theErrLog) << "-d value should be an integer" << endl;
       return false;
     }
@@ -196,10 +207,17 @@ bool parse_args( TimblOpts& Opts ) {
   if ( Opts.Find ('n', value, mood)) {
     doSentencePerLine = true;
   };
+  if ( Opts.Find ('Q', value, mood)) {
+    doQuoteDetection = true;
+  };
   if ( Opts.Find( "skip", value, mood)) {
     string skip = value;
     if ( skip.find_first_of("tT") != string::npos )
       doTok = false;
+    if ( skip.find_first_of("lL") != string::npos )
+      doLemma = false;
+    if ( skip.find_first_of("aA") != string::npos )
+      doMorph = false;
     if ( skip.find_first_of("mM") != string::npos )
       doMwu = false;
     if ( skip.find_first_of("cC") != string::npos )
@@ -221,17 +239,37 @@ bool parse_args( TimblOpts& Opts ) {
       return false;
     }
     else {
-      if ( !Timbl::stringTo<unsigned int>( value, maxParserTokens ) ){
+      if ( !stringTo<unsigned int>( value, maxParserTokens ) ){
 	*Log(theErrLog) << "max-parser-tokens value should be an integer" << endl;
 	return false;
       }
     }
     Opts.Delete("max-parser-tokens");
   }
+
+  if ( Opts.Find ('S', value, mood)) {
+    doServer = true;
+    listenport= value;
+  }
+#ifdef HAVE_OPENMP      
+  if ( doServer ) {
+    // run in one thread in server mode, forking is too expensive for lots of small snippets
+    omp_set_num_threads( 1 );
+  }
+  else if ( Opts.Find( "threads", value, mood ) ){
+    int num;
+    if ( !stringTo<int>( value, num ) || num < 1 ){
+      *Log(theErrLog) << "threads value should be a positive integer" << endl;
+      return false;
+    }
+    omp_set_num_threads( num );
+  }
+#endif
+
   if ( Opts.Find( "keep-parser-files", value, mood ) ){
     if ( value.empty() ||
 	 value == "true" || value == "TRUE" || value =="yes" || value == "YES" )
-      configuration.set( "keepIntermediateFiles", "true", "parser" );
+      configuration.setatt( "keepIntermediateFiles", "true", "parser" );
     Opts.Delete("keep-parser-files");
   }
   tmpDirName = configuration.lookUp( "tmpdir", "global" );
@@ -263,12 +301,6 @@ bool parse_args( TimblOpts& Opts ) {
 	*Log(theErrLog) << "input dir " << testDirName << " not readable" << endl;
 	return false;
       }
-      getFileNames( testDirName, fileNames );
-      if ( fileNames.empty() ){
-	*Log(theErrLog) << "error: couln't find any files in directory: " 
-	     << testDirName << endl;
-	return false;
-      }
     }
     else {
       *Log(theErrLog) << "empty testdir name!" << endl;
@@ -330,7 +362,10 @@ bool parse_args( TimblOpts& Opts ) {
     XMLoutFileName = value;
     Opts.Delete('X');
   }
-  
+  if ( Opts.Find ("KANON", value, mood) ){
+    doKanon = true;
+    Opts.Delete( "KANON" );
+  }
   doXMLin = false;
   if ( Opts.Find ('x', value, mood)) {
     doXMLin = true;
@@ -350,6 +385,14 @@ bool parse_args( TimblOpts& Opts ) {
     }
     Opts.Delete('x');
   }
+  if ( Opts.Find ( "textclass", value, mood)) {
+    if ( !doXMLin ){
+      *Log(theErrLog) << "--textclass is only valid when -x is also present" << endl;
+      return false;
+    }
+    textclass = value;
+    Opts.Delete( "textclass");
+  }  
   
   if ( !outputFileName.empty() && !testDirName.empty() ){
     *Log(theErrLog) << "useless -o value" << endl;
@@ -359,16 +402,25 @@ bool parse_args( TimblOpts& Opts ) {
     *Log(theErrLog) << "useless -X value" << endl;
     return false;
   }
-
-  if ( Opts.Find ('S', value, mood)) {
-    doServer = true;
-    listenport= value;
+  
+  if ( Opts.Find ("uttmarker", value, mood)) {
+    uttmark = value;
   }
   if ( !outputDirName.empty() && testDirName.empty() ){
     *Log(theErrLog) << "useless -outputdir option" << endl;
     return false;
   }
-
+  if ( !testDirName.empty() ){
+    if ( doXMLin )
+      getFileNames( testDirName, ".xml", fileNames );
+    else
+      getFileNames( testDirName, "", fileNames );
+    if ( fileNames.empty() ){
+      *Log(theErrLog) << "error: couln't find any files in directory: " 
+		      << testDirName << endl;
+      return false;
+    }
+  }
   if ( !doServer && TestFileName.empty() && fileNames.empty() ){
     *Log(theErrLog) << "no frogging without input!" << endl;
     return false;
@@ -410,24 +462,29 @@ bool froginit(){
     bool stat = tokenizer.init( configuration, docid, !doTok );
     if ( stat ){
       tokenizer.setSentencePerLineInput( doSentencePerLine );
+      tokenizer.setQuoteDetection( doQuoteDetection );
       tokenizer.setInputEncoding( encoding );
       tokenizer.setInputXml( doXMLin );
+      tokenizer.setUttMarker( uttmark );
+      tokenizer.setTextClass( textclass );
       stat = myCGNTagger.init( configuration );
       if ( stat ){
-	if ( doIOB )
+	if ( doIOB ){
 	  stat = myIOBTagger.init( configuration );
-	if ( stat && doNER )
+	}
+	if ( stat && doNER ){
 	  stat = myNERTagger.init( configuration );
-	if ( stat ){
+	}
+	if ( stat && doLemma ){
 	  stat = myMblem.init( configuration );
-	  if ( stat ){
-	    stat = myMbma.init( configuration );
-	    if ( stat ) {
-	      if ( doMwu ){
-		stat = myMwu.init( configuration );
-		if ( stat && doParse ){
-		  stat = myParser.init( configuration );
-		}
+	}
+	if ( stat && doMorph ){
+	  stat = myMbma.init( configuration );
+	  if ( stat ) {
+	    if ( doMwu ){
+	      stat = myMwu.init( configuration );
+	      if ( stat && doParse ){
+		stat = myParser.init( configuration );
 	      }
 	    }
 	  }
@@ -455,17 +512,24 @@ bool froginit(){
 	tokStat = tokenizer.init( configuration, docid, !doTok );
 	if ( tokStat ){
 	  tokenizer.setSentencePerLineInput( doSentencePerLine );
+	  tokenizer.setQuoteDetection( doQuoteDetection );
 	  tokenizer.setInputEncoding( encoding );
 	  tokenizer.setInputXml( doXMLin );
+	  tokenizer.setUttMarker( uttmark );
+	  tokenizer.setTextClass( textclass );
 	}
       }
 #pragma omp section
       {
-	lemStat = myMblem.init( configuration );
+	if ( doLemma ){
+	  lemStat = myMblem.init( configuration );
+	}
       }
 #pragma omp section
       {
-	mbaStat = myMbma.init( configuration );
+	if ( doMorph ){
+	  mbaStat = myMbma.init( configuration );
+	}
       }
 #pragma omp section 
       {
@@ -473,20 +537,22 @@ bool froginit(){
       }
 #pragma omp section 
       {
-	if ( doIOB )
+	if ( doIOB ){
 	  iobStat = myIOBTagger.init( configuration );
+	}
       }
 #pragma omp section 
       {
-	if ( doNER )
+	if ( doNER ){
 	  nerStat = myNERTagger.init( configuration );
+	}
       }
 #pragma omp section
       {
 	if ( doMwu ){
 	  mwuStat = myMwu.init( configuration );
 	  if ( mwuStat && doParse ){
-	    Common::Timer initTimer;
+	    Timer initTimer;
 	    initTimer.start();
 	    parStat = myParser.init( configuration );
 	    initTimer.stop();
@@ -547,16 +613,23 @@ vector<Word*> lookup( Word *word, const vector<Entity*>& entities ){
 
 Dependency *lookupDep( const Word *word, 
 		       const vector<Dependency*>&dependencies ){
-  //  cerr << "\nlookup "<< word << " in " << dependencies << endl;
+  if ( tpDebug ){
+    using TiCC::operator<<;
+    *Log( theErrLog ) << "lookup "<< word << " in " << dependencies << endl;
+  }
   for ( size_t i=0; i < dependencies.size(); ++i ){
-    //    cerr << "\nprobeer " << dependencies[i] << endl;
+    if ( tpDebug ){
+      *Log( theErrLog ) << "probeer " << dependencies[i] << endl;
+    }
     try {
       vector<DependencyDependent*> dv = dependencies[i]->select<DependencyDependent>();
       if ( !dv.empty() ){
 	vector<Word*> v = dv[0]->select<Word>();
 	for ( size_t j=0; j < v.size(); ++j ){
 	  if ( v[j] == word ){
-	    //	    cerr << "\nfound word " << v[j] << endl;
+	    if ( tpDebug ){
+	      *Log(theErrLog) << "\nfound word " << v[j] << endl;
+	    }
 	    return dependencies[i];
 	  }
 	}
@@ -576,7 +649,7 @@ string lookupNEREntity( const vector<Word *>& mwu,
   string endresult;
   for ( size_t j=0; j < mwu.size(); ++j ){
     if ( tpDebug ){
-      using folia::operator<<;
+      using TiCC::operator<<;
       *Log(theErrLog) << "lookup "<< mwu[j] << " in " << entities << endl;
     }
     string result;
@@ -593,9 +666,9 @@ string lookupNEREntity( const vector<Word *>& mwu,
 	      *Log(theErrLog) << "found word " << v[k] << endl;
 	    }
 	    if ( first )
-	      result += "B-" + TiCC::uppercase(entities[i]->cls());
+	      result += "B-" + uppercase(entities[i]->cls());
 	    else
-	      result += "I-" + TiCC::uppercase(entities[i]->cls());
+	      result += "I-" + uppercase(entities[i]->cls());
 	    break;
 	  }
 	  else
@@ -624,7 +697,7 @@ string lookupIOBChunk( const vector<Word *>& mwu,
   string endresult;
   for ( size_t j=0; j < mwu.size(); ++j ){
     if ( tpDebug ){
-      using folia::operator<<;
+      using TiCC::operator<<;
       *Log(theErrLog) << "lookup "<< mwu[j] << " in " << chunks << endl;
     }
     string result;
@@ -677,7 +750,7 @@ void displayMWU( ostream& os, size_t index,
     Word *word = mwu[p];
     try { 
       wrd += word->str();
-      PosAnnotation *postag = word->annotation<PosAnnotation>();
+      PosAnnotation *postag = word->annotation<PosAnnotation>( );
       pos += postag->cls();
       if ( p < mwu.size() -1 ){
 	wrd += "_";
@@ -729,12 +802,24 @@ ostream &showResults( ostream& os,
 		      bool showParse ){
   vector<Word*> words = sentence->words();
   vector<Entity*> mwu_entities = sentence->select<Entity>( myMwu.getTagset() );
-  // using folia::operator<<;
-  // *Log(theErrLog) << "mwu entities " << mwu_entities << endl;
   vector<Dependency*> dependencies = sentence->select<Dependency>();
   vector<Chunk*> iob_chunking = sentence->select<Chunk>();
   vector<Entity*> ner_entities = sentence->select<Entity>( myNERTagger.getTagset() );
-  //  *Log(theErrLog) << "ner entities " << ner_entities << endl;
+  static set<ElementType> excludeSet;
+  vector<Sentence*> parts = sentence->select<Sentence>( excludeSet );
+  if ( !doQuoteDetection )
+    assert( parts.size() == 0 );
+  for ( size_t i=0; i < parts.size(); ++i ){
+    vector<Entity*> ents = parts[i]->select<Entity>( myMwu.getTagset() );
+    mwu_entities.insert( mwu_entities.end(), ents.begin(), ents.end() );
+    vector<Dependency*> deps = parts[i]->select<Dependency>();
+    dependencies.insert( dependencies.end(), deps.begin(), deps.end() );
+    vector<Chunk*> chunks = parts[i]->select<Chunk>();
+    iob_chunking.insert( iob_chunking.end(), chunks.begin(), chunks.end() );
+    vector<Entity*> ners = parts[i]->select<Entity>( myNERTagger.getTagset() );
+    ner_entities.insert( ner_entities.end(), ners.begin(), ners.end() );
+  }
+
   size_t index = 1;
   map<FoliaElement*, int> enumeration;
   vector<vector<Word*> > mwus;
@@ -770,8 +855,17 @@ ostream &showResults( ostream& os,
       string cls;
       Dependency *dep = lookupDep( mwus[i][0], dependencies );
       if ( dep ){
-	vector<DependencyHead*> w = dep->select<DependencyHead>();
-	os << "\t" << enumeration.find(w[0]->index(0))->second << "\t" << dep->cls();
+	vector<Headwords*> w = dep->select<Headwords>();
+	size_t num;
+	if ( w[0]->index(0)->isinstance( PlaceHolder_t ) ){
+	  string indexS = w[0]->index(0)->str();
+	  FoliaElement *pnt = w[0]->index(0)->doc()->index(indexS);
+	  num = enumeration.find(pnt->index(0))->second;
+	}
+	else {
+	  num = enumeration.find(w[0]->index(0))->second;
+	}
+	os << "\t" << num << "\t" << dep->cls();
       }
       else {
 	os << "\t"<< 0 << "\tROOT";
@@ -788,12 +882,14 @@ ostream &showResults( ostream& os,
   return os;
 }
 
-void TestSentence( const vector<Sentence*>& sentences,
-		   size_t index,
+bool TestSentence( Sentence* sent,
 		   ostream& outStream,
 		   TimerBlock& timers ){
-  Sentence *sent = sentences[index];
-  vector<Word*> swords = sent->words();
+  vector<Word*> swords;
+  if ( doQuoteDetection )
+    swords = sent->wordParts();
+  else
+    swords = sent->words();
   bool showParse = doParse;
   if ( !swords.empty() ) {
 #pragma omp parallel sections
@@ -801,14 +897,14 @@ void TestSentence( const vector<Sentence*>& sentences,
 #pragma omp section
       {
 	timers.tagTimer.start();
-	myCGNTagger.Classify( sent );
+	myCGNTagger.Classify( swords );
 	timers.tagTimer.stop();
       }
 #pragma omp section
       {
 	if ( doIOB ){
 	  timers.iobTimer.start();
-	  myIOBTagger.Classify( sent );
+	  myIOBTagger.Classify( swords );
 	  timers.iobTimer.stop();
 	}
       }
@@ -816,60 +912,54 @@ void TestSentence( const vector<Sentence*>& sentences,
       {
 	if ( doNER ){
 	  timers.nerTimer.start();
-	  myNERTagger.Classify( sent );
+	  myNERTagger.Classify( swords );
 	  timers.nerTimer.stop();
 	}
       }
     } // parallel sections
     for ( size_t i = 0; i < swords.size(); ++i ) {
-      string mbmaLemma;
-      string mblemLemma;
 #pragma omp parallel sections
       {
 #pragma omp section
 	{
-	  timers.mbmaTimer.start();
-	  if (tpDebug) 
-	    *Log(theErrLog) << "Calling mbma..." << endl;
-	  mbmaLemma = myMbma.Classify( swords[i] );
-	  timers.mbmaTimer.stop();
+	  if ( doMorph ){
+	    timers.mbmaTimer.start();
+	    if (tpDebug) 
+	      *Log(theErrLog) << "Calling mbma..." << endl;
+	    myMbma.Classify( swords[i] );
+	    timers.mbmaTimer.stop();
+	  }
 	}
 #pragma omp section
 	{
-	  timers.mblemTimer.start();
-	  if (tpDebug) 
-	    *Log(theErrLog) << "Calling mblem..." << endl;
-	  mblemLemma = myMblem.Classify( swords[i] );
-	  timers.mblemTimer.stop();
+	  if ( doLemma ){
+	    timers.mblemTimer.start();
+	    if (tpDebug) 
+	      *Log(theErrLog) << "Calling mblem..." << endl;
+	    myMblem.Classify( swords[i] );
+	    timers.mblemTimer.stop();
+	  }
 	}
       } // omp parallel sections
-      if (tpDebug) {
-	*Log(theErrLog) << "tagged word[" << i <<"]: " << swords[i]->str() 
-			<< " tag " << swords[i]->pos() << endl;
-	*Log(theErrLog) << "analysis: " << mblemLemma << " " << mbmaLemma << endl;
-      }
     } //for int i = 0 to num_words
     
     if ( doMwu ){
       if ( swords.size() > 0 ){
 	timers.mwuTimer.start();
-	myMwu.Classify( sent );
+	myMwu.Classify( swords );
 	timers.mwuTimer.stop();
       }
     }
     if ( doParse ){
       if ( maxParserTokens != 0 && swords.size() > maxParserTokens ){
-	*Log(theErrLog) << "WARNING!" << endl;
-	*Log(theErrLog) << "Sentence " << index+1 << " isn't parsed because it contains " << swords.size() << " tokens, and you provided the --max-parser-tokens=" << maxParserTokens << " option." << endl;
 	showParse = false;
       }
       else {
-	myParser.Parse( sent, myMwu.getTagset(), tmpDirName, timers );
+	myParser.Parse( swords, myMwu.getTagset(), tmpDirName, timers );
       }
     }
   }
-  if ( !(doServer && doXMLout) )
-    showResults( outStream, sent, showParse );
+  return showParse;
 }
 
 void Test( Document& doc,
@@ -880,8 +970,10 @@ void Test( Document& doc,
   // first we make sure that the doc will accept our annotations, by
   // declaring them in the doc
   myCGNTagger.addDeclaration( doc );
-  myMblem.addDeclaration( doc );
-  myMbma.addDeclaration( doc );
+  if ( doLemma )
+    myMblem.addDeclaration( doc );
+  if ( doMorph )
+    myMbma.addDeclaration( doc );
   if (doIOB)
     myIOBTagger.addDeclaration( doc );
   if (doNER)
@@ -890,22 +982,37 @@ void Test( Document& doc,
     myMwu.addDeclaration( doc );
   if (doParse) 
     myParser.addDeclaration( doc );
+
   if ( tpDebug > 5 )
     *Log(theErrLog) << "Testing document :" << doc << endl;
-  vector<Sentence*> sentences = doc.sentences();
+  
+  vector<Sentence*> topsentences = doc.sentences();
+  vector<Sentence*> sentences;
+  if ( doQuoteDetection )
+    sentences = doc.sentenceParts();
+  else
+    sentences = topsentences;
   size_t numS = sentences.size();
   if ( numS > 0 ) { //process sentences 
     if  (tpDebug > 0) 
       *Log(theErrLog) << "found " << numS << " sentence(s) in document." << endl;
     for ( size_t i = 0; i < numS; i++) {
       /* ******* Begin process sentence  ********** */
-      TestSentence( sentences, i, outStream, timers ); 
       //NOTE- full sentences are passed (which may span multiple lines) (MvG)
+      bool showParse = TestSentence( sentences[i], outStream, timers ); 
+      if ( doParse && !showParse ){
+	*Log(theErrLog) << "WARNING!" << endl;
+	*Log(theErrLog) << "Sentence " << i+1 << " isn't parsed because it contains more tokens then set with the --max-parser-tokens=" << maxParserTokens << " option." << endl;
+      }
+    }
+    for ( size_t i = 0; i < topsentences.size(); ++i ) {
+      if ( !(doServer && doXMLout) )
+	showResults( outStream, topsentences[i], doParse );
     }
     if ( doServer && doXMLout )
       outStream << doc << endl;
     if ( !xmlOutFile.empty() ){
-      doc.save( xmlOutFile );
+      doc.save( xmlOutFile, doKanon );
       *Log(theErrLog) << "resulting FoLiA doc saved in " << xmlOutFile << endl;
     }
   } 
@@ -922,8 +1029,10 @@ void Test( Document& doc,
     *Log(theErrLog) << "IOB chunking took:  " << timers.iobTimer << endl;
   if ( doNER)
     *Log(theErrLog) << "NER took:           " << timers.nerTimer << endl;
-  *Log(theErrLog) << "MBA took:           " << timers.mbmaTimer << endl;
-  *Log(theErrLog) << "Mblem took:         " << timers.mblemTimer << endl;
+  if ( doMorph )
+    *Log(theErrLog) << "MBA took:           " << timers.mbmaTimer << endl;
+  if ( doLemma )
+    *Log(theErrLog) << "Mblem took:         " << timers.mblemTimer << endl;
   if ( doMwu )
     *Log(theErrLog) << "MWU resolving took: " << timers.mwuTimer << endl;
   if ( doParse ){
@@ -954,9 +1063,19 @@ void Test( const string& infilename,
       exit( EXIT_FAILURE );
     }
   }
+  *Log(theErrLog) << "Frogging " << infilename << endl;
   if ( doXMLin ){
     Document doc;
-    doc.readFromFile( infilename );
+    try {
+      doc.readFromFile( infilename );
+    }
+    catch ( exception &e ){
+      *Log(theErrLog) << "retrieving FoLiA from '" << infilename << "' failed" << endl;
+      if ( !outName.empty() ){
+	delete os;
+      }
+      return;
+    }
     tokenizer.tokenize( doc );
     Test( doc, *os, xmlOutFile );
   }
@@ -1006,9 +1125,19 @@ void TestServer( Sockets::ServerSocket &conn) {
 	Test( doc, outputstream );
       }
       else {
-	string data = "";      
-	if ( !conn.read( data ) )	 //read data from client
-	  throw( runtime_error( "read failed" ) );
+	string data = "";
+	if ( doSentencePerLine ){
+	  if ( !conn.read( data ) )	 //read data from client
+	    throw( runtime_error( "read failed" ) );
+	}
+	else {
+	  string line;
+	  while( conn.read(line) ){
+	    if ( line == "EOT" )
+	      break;
+	    data += line + "\n";
+	  }
+	}
 	if (tpDebug)
 	  *Log(theErrLog) << "Received: [" << data << "]" << endl;
 	*Log(theErrLog) << "Processing... " << endl;
@@ -1033,8 +1162,7 @@ void TestServer( Sockets::ServerSocket &conn) {
 
 
 int main(int argc, char *argv[]) {
-  std::ios_base::sync_with_stdio(false);
-  cerr << "frog " << VERSION << " (c) ILK 1998 - 2012" << endl;
+  cerr << "frog " << VERSION << " (c) ILK 1998 - 2013" << endl;
   cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
   ProgName = argv[0];
   cerr << "based on [" << Tokenizer::VersionName() << ", "
@@ -1042,19 +1170,14 @@ int main(int argc, char *argv[]) {
        << Timbl::VersionName() << ", "
        << TimblServer::VersionName() << ", "
        << Tagger::VersionName() << "]" << endl;
-  cout << "configdir: " << configDir << endl;
+  //  cout << "configdir: " << configDir << endl;
+  std::ios_base::sync_with_stdio(false);
   try {
     TimblOpts Opts(argc, argv);
         
     if ( parse_args(Opts) ){
       if (  !froginit() ){
-	cerr << "terminated." << endl;
-	return EXIT_FAILURE;
-      }
-      
-      if ( doServer ) {
-      	// run in one thread in server mode, forking is too expensive for lots of small snippets
-      	omp_set_num_threads( 1 );
+	throw runtime_error( "init failed" );
       }
       if ( !fileNames.empty() ) {
 	string outPath;
@@ -1118,7 +1241,7 @@ int main(int argc, char *argv[]) {
 		int pid = fork();				
 		if (pid < 0) {
 		  *Log(theErrLog) << "ERROR on fork" << endl;
-		  exit(EXIT_FAILURE);
+		  throw runtime_error( "FORK failed" );
 		} else if (pid == 0)  {
 		  //		  server = NULL;
 		  TestServer(conn );
@@ -1132,7 +1255,7 @@ int main(int argc, char *argv[]) {
 	  } catch ( std::exception& e )
 	  {
 	    *Log(theErrLog) << "Server error:" << e.what() << " Exiting." << endl;
-	    exit(EXIT_FAILURE);
+	    throw;
 	  }
 	
       } else {
@@ -1144,14 +1267,12 @@ int main(int argc, char *argv[]) {
       }
     }
     else {
-      return EXIT_FAILURE;
+      throw runtime_error( "invalid arguments" );
     }
   }
   catch ( const exception& e ){
     *Log(theErrLog) << "fatal error: " << e.what() << endl;
     return EXIT_FAILURE;
   }
-  //  delete tagger;
-  
   return EXIT_SUCCESS;
 }
