@@ -1,8 +1,8 @@
 /*
-  $Id: Parser.cxx 14481 2012-03-21 11:26:55Z sloot $
+  $Id: Parser.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Parser.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -39,15 +39,13 @@
 
 #include "config.h"
 #include "timbl/TimblAPI.h"
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
 #include "frog/Frog.h"
-#include "frog/Configuration.h"
+#include "ticcutils/Configuration.h"
 #include "frog/Parser.h"
 
 using namespace std;
-using namespace folia;
 using namespace TiCC;
+using namespace folia;
 
 PythonInterface::PythonInterface( ) {
   Py_OptimizeFlag = 1; // enable optimisation (-O) mode
@@ -117,11 +115,11 @@ void PythonInterface::parse( const string& depFile,
 					  inputFile.c_str() );
 }
 
-Common::Timer prepareTimer;
-Common::Timer relsTimer;
-Common::Timer pairsTimer;
-Common::Timer dirTimer;
-Common::Timer csiTimer;
+TiCC::Timer prepareTimer;
+TiCC::Timer relsTimer;
+TiCC::Timer pairsTimer;
+TiCC::Timer dirTimer;
+TiCC::Timer csiTimer;
 
 struct parseData {
   void clear() { words.clear(); heads.clear(); mods.clear(); mwus.clear(); }
@@ -190,9 +188,17 @@ bool Parser::init( const Configuration& configuration ){
   }
   else
     tagset = val;
+
+  val = configuration.lookUp( "set", "tagger" );
+  if ( val.empty() ){
+    cgn_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
+  }
+  else
+    cgn_tagset = val;
+
   val = configuration.lookUp( "maxDepSpan", "parser" );
   if ( !val.empty() ){
-    size_t gs = stringTo<size_t>( val );
+    size_t gs = TiCC::stringTo<size_t>( val );
     if ( gs < 50 ){
       maxDepSpanS = val;
       maxDepSpan = gs;
@@ -768,21 +774,22 @@ void Parser::addDeclaration( Document& doc ) const {
 	       + "', annotatortype='auto'");
 }
 
-void Parser::prepareParse( Sentence *sent, const string& setname,
+void Parser::prepareParse( const vector<Word *>& fwords,
+			   const string& setname,
 			   parseData& pd ) {
   pd.clear();
-  vector<Word*> fwords = sent->words();
+  Sentence *sent = fwords[0]->sentence();
   vector<Entity*> entities = sent->select<Entity>(setname);
   for( size_t i=0; i < fwords.size(); ++i ){
     Word *word = fwords[i];
     vector<Word*> mwu = lookup( word, entities );
     if ( !mwu.empty() ){
-      string word;
+      string multi_word;
       string head;
       string mod;
       for ( size_t p=0; p < mwu.size(); ++p ){
-	word += mwu[p]->str();
-	PosAnnotation *postag = mwu[p]->annotation<PosAnnotation>();
+	multi_word += mwu[p]->str();
+	PosAnnotation *postag = mwu[p]->annotation<PosAnnotation>( cgn_tagset );
 	head += postag->feat("head");
 	vector<folia::Feature*> feats = postag->select<folia::Feature>();
 	for ( size_t j=0; j < feats.size(); ++j ){
@@ -791,12 +798,12 @@ void Parser::prepareParse( Sentence *sent, const string& setname,
 	    mod += "|";
 	}
 	if ( p < mwu.size() -1 ){
-	  word += "_";
+	  multi_word += "_";
 	  head += "_";
 	  mod += "_";
 	}
       }
-      pd.words.push_back( word );
+      pd.words.push_back( multi_word );
       pd.heads.push_back( head );
       pd.mods.push_back( mod );
       pd.mwus.push_back( mwu );
@@ -804,7 +811,7 @@ void Parser::prepareParse( Sentence *sent, const string& setname,
     }
     else {
       pd.words.push_back( word->str() );
-      PosAnnotation *postag = word->annotation<PosAnnotation>();
+      PosAnnotation *postag = word->annotation<PosAnnotation>( cgn_tagset );
       string head = postag->feat("head");
       pd.heads.push_back( head );
       string mod;
@@ -840,57 +847,59 @@ void Parser::prepareParse( Sentence *sent, const string& setname,
   }
 }
 
- void appendParseResult( Sentence *sent, 
-			 parseData& pd,
-			 istream& is ){
-   string line;
-   int cnt=0;
-   vector<int> nums;
-   vector<string> roles;
-   while ( getline( is, line ) ){
-     vector<string> parts;
-     int num = TiCC::split_at( line, parts, " " );
-     if ( num > 7 ){
-       if ( Timbl::stringTo<int>( parts[0] ) != cnt+1 ){
-	 *Log(theErrLog) << "confused! " << endl;
-	 *Log(theErrLog) << "got line '" << line << "'" << endl;
-       }
-       nums.push_back( Timbl::stringTo<int>(parts[6]) );
-       roles.push_back( parts[7] );
-     }
-     ++cnt;
-   }
-   DependenciesLayer *dl = new DependenciesLayer();
+void appendParseResult( const vector<Word *>& words,
+			parseData& pd,
+			istream& is ){
+  string line;
+  int cnt=0;
+  vector<int> nums;
+  vector<string> roles;
+  while ( getline( is, line ) ){
+    vector<string> parts;
+    int num = TiCC::split_at( line, parts, " " );
+    if ( num > 7 ){
+      if ( TiCC::stringTo<int>( parts[0] ) != cnt+1 ){
+	*Log(theErrLog) << "confused! " << endl;
+	*Log(theErrLog) << "got line '" << line << "'" << endl;
+      }
+      nums.push_back( TiCC::stringTo<int>(parts[6]) );
+      roles.push_back( parts[7] );
+    }
+    ++cnt;
+  }
+  DependenciesLayer *dl = new DependenciesLayer();
+  Sentence *sent;
 #pragma omp critical(foliaupdate)
-   {
-     sent->append( dl );
-   }
-   for ( size_t i=0; i < nums.size(); ++i ){
-     if ( nums[i] != 0 ){
-       KWargs args;
-       args["generate-id"] = sent->id();
-       args["class"] = roles[i];
+  {
+    sent = words[0]->sentence();
+    sent->append( dl );
+  }
+  for ( size_t i=0; i < nums.size(); ++i ){
+    if ( nums[i] != 0 ){
+      KWargs args;
+      args["generate-id"] = sent->id();
+      args["class"] = roles[i];
 #pragma omp critical(foliaupdate)
-       {
-	 Dependency *d = new Dependency( sent->doc(), args );
-	 dl->append( d );
-	 DependencyHead *dh = new DependencyHead();
-	 for ( size_t j=0; j < pd.mwus[nums[i]-1].size(); ++ j ){
-	   dh->append( pd.mwus[nums[i]-1][j] );
-	 }
-	 d->append( dh );
-	 DependencyDependent *dd = new DependencyDependent();
-	 for ( size_t j=0; j < pd.mwus[i].size(); ++ j ){
-	   dd->append( pd.mwus[i][j] );
-	 }
-	 d->append( dd );
-       }
-     }
-   }
- }
- 
- 
-void Parser::Parse( Sentence *sent, const string& mwuSet,
+      {
+	Dependency *d = new Dependency( sent->doc(), args );
+	dl->append( d );
+	Headwords *dh = new Headwords();
+	for ( size_t j=0; j < pd.mwus[nums[i]-1].size(); ++ j ){
+	  dh->append( pd.mwus[nums[i]-1][j] );
+	}
+	d->append( dh );
+	DependencyDependent *dd = new DependencyDependent();
+	for ( size_t j=0; j < pd.mwus[i].size(); ++ j ){
+	  dd->append( pd.mwus[i][j] );
+	}
+	d->append( dd );
+      }
+    }
+  }
+}
+
+
+void Parser::Parse( const vector<Word*>& words, const string& mwuSet,
 		    const string& tmpDirName, TimerBlock& timers ){
   pid_t pid = getpid();
   string pids = toString( pid );
@@ -900,7 +909,7 @@ void Parser::Parse( Sentence *sent, const string& mwuSet,
     *Log(parseLog) << "Parser is not initialized!" << endl;
     exit(1);
   }
-  if ( !sent ){
+  if ( words.empty() ){
     *Log(parseLog) << "unable to parse an analisis without words" << endl;
     return;
   }
@@ -914,7 +923,7 @@ void Parser::Parse( Sentence *sent, const string& mwuSet,
   remove( resFileName.c_str() );
   timers.prepareTimer.start();
   parseData pd;  
-  prepareParse( sent, mwuSet, pd );
+  prepareParse( words, mwuSet, pd );
   timers.prepareTimer.stop();
 #pragma omp parallel sections
   {
@@ -957,7 +966,7 @@ void Parser::Parse( Sentence *sent, const string& mwuSet,
   timers.csiTimer.stop();
   ifstream resFile( resFileName.c_str() );
   if ( resFile ){
-    appendParseResult( sent, pd, resFile );
+    appendParseResult( words, pd, resFile );
   }
   else
     *Log(parseLog) << "couldn't open results file: " << resFileName << endl;
