@@ -1,8 +1,8 @@
 /*
-  $Id: iob_tagger_mod.cxx 14481 2012-03-21 11:26:55Z sloot $
+  $Id: iob_tagger_mod.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/iob_tagger_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -29,8 +29,6 @@
 */
 
 #include "mbt/MbtAPI.h"
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
 #include "frog/Frog.h"
 #include "frog/iob_tagger_mod.h"
 
@@ -51,7 +49,7 @@ bool IOBTagger::init( const Configuration& conf ){
   debug = tpDebug;
   string db = conf.lookUp( "debug", "IOB" );
   if ( !db.empty() )
-    debug = folia::stringTo<int>( db );
+    debug = TiCC::stringTo<int>( db );
   switch ( debug ){
   case 0:
   case 1:
@@ -81,7 +79,12 @@ bool IOBTagger::init( const Configuration& conf ){
     *Log(iobLog) << "Unable to find settings for IOB" << endl;
     return false;
   }
-  string settings = val;
+  string settings;
+  if ( val[0] == '/' ) // an absolute path
+    settings = val;
+  else
+    settings =  configuration.configDir() + val;
+
   val = conf.lookUp( "version", "IOB" );
   if ( val.empty() ){
     version = "1.0";
@@ -95,9 +98,9 @@ bool IOBTagger::init( const Configuration& conf ){
   else
     tagset = val;
 
-  string init = "-s " + configuration.configDir() + settings + " -vcf";
+  string init = "-s " + settings + " -vcf";
   tagger = new MbtAPI( init, *iobLog );
-  return tagger != 0;
+  return tagger->isInit();
 }
 
 void IOBTagger::addChunk( ChunkingLayer *chunks, 
@@ -124,6 +127,8 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
     }
   }
   for ( size_t i=0; i < words.size(); ++i ){
+    if ( words[i]->isinstance(PlaceHolder_t) )
+      continue;
 #pragma omp critical(foliaupdate)
     {
       chunk->append( words[i] );
@@ -131,13 +136,15 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
   }
 }
 
-void IOBTagger::addIOBTags( Sentence *sent, 
-			    const vector<Word*>& words,
+void IOBTagger::addIOBTags( const vector<Word*>& words,
 			    const vector<string>& tags,
 			    const vector<double>& confs ){
+  if ( words.empty() )
+    return;
   ChunkingLayer *el = 0;
 #pragma omp critical(foliaupdate)
   {
+    Sentence *sent = words[0]->sentence();
     try {
       el = sent->annotation<ChunkingLayer>();
     }
@@ -165,7 +172,7 @@ void IOBTagger::addIOBTags( Sentence *sent,
       if ( !stack.empty() ){
 	if (debug) {
 	  *Log(iobLog) << "O spit out " << curIOB << endl;
-	  using folia::operator<<;
+	  using TiCC::operator<<;
 	  *Log(iobLog) << "spit out " << stack << endl;	
 	}
 	addChunk( el, stack, dstack, curIOB );
@@ -189,7 +196,7 @@ void IOBTagger::addIOBTags( Sentence *sent,
       if ( !stack.empty() ){
 	if ( debug ){
 	  *Log(iobLog) << "B spit out " << curIOB << endl;
-	  using folia::operator<<;
+	  using TiCC::operator<<;
 	  *Log(iobLog) << "spit out " << stack << endl;	
 	}
 	addChunk( el, stack, dstack, curIOB );
@@ -204,7 +211,7 @@ void IOBTagger::addIOBTags( Sentence *sent,
   if ( !stack.empty() ){
     if ( debug ){
       *Log(iobLog) << "END spit out " << curIOB << endl;
-      using folia::operator<<;
+      using TiCC::operator<<;
       *Log(iobLog) << "spit out " << stack << endl;	
     }
     addChunk( el, stack, dstack, curIOB );
@@ -215,15 +222,10 @@ void IOBTagger::addDeclaration( Document& doc ) const {
   doc.declare( AnnotationType::CHUNKING, 
 	       tagset,
 	       "annotator='frog-chunker-" + version
-	       + "', annotatortype='auto'");
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
 
-void IOBTagger::Classify( Sentence *sent ){
-  vector<Word*> swords;
-#pragma omp critical(foliaupdate)
-  {  
-    swords = sent->words();
-  }
+void IOBTagger::Classify( const vector<Word *>& swords ){
   if ( !swords.empty() ) {
     vector<string> words;
     string sentence; // the tagger needs the whole sentence
@@ -253,7 +255,7 @@ void IOBTagger::Classify( Sentence *sent ){
       tags.push_back( tagv[i].assignedTag() );
       conf.push_back( tagv[i].confidence() );
     }
-    addIOBTags( sent, swords, tags, conf );
+    addIOBTags( swords, tags, conf );
   }
 }
 
