@@ -1,8 +1,8 @@
 /*
-  $Id: mblem_mod.cxx 14525 2012-03-27 09:07:23Z sloot $
+  $Id: mblem_mod.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mblem_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -31,17 +31,14 @@
 #include <iostream>
 #include <fstream>
 #include "timbl/TimblAPI.h"
-
-#include "ucto/unicode.h"
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
+#include "ticcutils/LogStream.h"
+#include "ticcutils/Configuration.h"
 #include "frog/Frog.h"
-#include "frog/Configuration.h"
 #include "frog/mblem_mod.h"
 
 using namespace std;
-using namespace folia;
 using namespace TiCC;
+using namespace folia;
 
 Mblem::Mblem(): myLex(0),punctuation( "?...,:;\\'`(){}[]%#+-_=/!" ), 
 		history(20), debug(0) {
@@ -79,7 +76,7 @@ bool Mblem::init( const Configuration& config ) {
   debug = tpDebug;
   string db = config.lookUp( "debug", "mblem" );
   if ( !db.empty() )
-    debug = stringTo<int>( db );
+    debug = TiCC::stringTo<int>( db );
   string val = config.lookUp( "version", "mblem" );
   if ( val.empty() ){
     version = "1.0";
@@ -92,6 +89,13 @@ bool Mblem::init( const Configuration& config ) {
   }
   else
     tagset = val;
+
+  val = config.lookUp( "set", "tagger" );
+  if ( val.empty() ){
+    cgn_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
+  }
+  else
+    cgn_tagset = val;
   
   string transName = config.lookUp( "transFile", "mblem" );
   if ( !transName.empty() ){
@@ -195,66 +199,106 @@ void Mblem::addAltLemma( Word *word, const string& cls ){
   addLemma( alt, cls );
 }
   
-string Mblem::postprocess( Word *word ){
-  string tag;
-#pragma omp critical(foliaupdate)
-  {
-    tag = word->pos();
-  }
-  size_t index = 0;
-  size_t nrlookup = mblemResult.size();
-  bool none=true;
-  string result;
-  while ( index < nrlookup  ){
+void Mblem::filterTag( const string& postag ){
+  vector<mblemData>::iterator it = mblemResult.begin();
+  while( it != mblemResult.end() ){
     if (debug)
-      *Log(mblemLog) << "compare cgn-tag " << tag << " with " << mblemResult[index].getTag() << endl;
-    if ( isSimilar( tag, mblemResult[index].getTag() ) ){
+      *Log(mblemLog) << "compare cgn-tag " << postag << " with " << it->getTag() << " ";
+    if ( isSimilar( postag, it->getTag() ) ){
       if ( debug )
 	*Log(mblemLog) << "similar! " << endl;
-      string res = mblemResult[index].getLemma();
-      if ( none ){
-	addLemma( word, res );
-	result = res;
-	none = false;
+      ++it;
+    }
+    else {
+      if ( debug )
+	*Log(mblemLog) << "NOT similar! " << endl;	
+      it = mblemResult.erase(it);
+    }
+  }
+  if ( debug && mblemResult.empty() )
+    *Log(mblemLog) << "NO CORRESPONDING TAG! " << postag << endl;
+}
+
+void Mblem::makeUnique( ){
+  vector<mblemData>::iterator it = mblemResult.begin();
+  while( it != mblemResult.end() ){
+    string lemma = it->getLemma();
+    vector<mblemData>::iterator it2 = it+1;
+    while( it2 != mblemResult.end() ){
+      if (debug)
+	*Log(mblemLog) << "compare lemma " << lemma << " with " << it2->getLemma() << " ";
+      if ( lemma == it2->getLemma() ){
+	if ( debug )
+	  *Log(mblemLog) << "equal " << endl;
+	it2 = mblemResult.erase(it2);
+      }
+      else {
+	if ( debug )
+	  *Log(mblemLog) << "NOT equal! " << endl;	
+	++it2;
+      }
+    }
+    ++it;
+  }
+}
+
+void Mblem::getFoLiAResult( Word *word, const UnicodeString& uWord ){
+  if ( mblemResult.empty() ){
+    // just return the word as a lemma
+    string result = UnicodeToUTF8( uWord );
+    addLemma( word, result );
+  }
+  else {
+    bool first = true;
+    vector<mblemData>::iterator it = mblemResult.begin();
+    while( it != mblemResult.end() ){
+      string result = it->getLemma();
+      if ( first ){
+	addLemma( word, result );
+	first = false;
       }
       else {
 	// there are more matching lemmas. add them as alternatives
-	addAltLemma( word, res );
+	addAltLemma( word, result );
       }
+      ++it;
     }
-    ++index;
   }
-  
-  if ( none  ) {
-    if (debug)
-      *Log(mblemLog) << "NO CORRESPONDING TAG! " << tag << endl;
-    result = mblemResult[0].getLemma();
-    addLemma( word, result );
-  }
-  return result;
 } 
 
+
 void Mblem::addDeclaration( Document& doc ) const {
   doc.declare( AnnotationType::LEMMA, 
 	       tagset,
 	       "annotator='frog-mblem-" + version
-	       + "', annotatortype='auto'");
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
 
-string Mblem::Classify( Word *sword ){
+void Mblem::Classify( Word *sword ){
+  if ( sword->isinstance(PlaceHolder_t ) )
+    return;
   string word;
   string tag;
+  string pos;
 #pragma omp critical(foliaupdate)
   {  
     word = sword->str();
-    tag = sword->annotation<PosAnnotation>()->feat("head");
+    pos = sword->pos();
+    tag = sword->annotation<PosAnnotation>( cgn_tagset )->feat("head");
   }
   if ( tag == "SPEC" || tag == "LET" ) {
     addLemma( sword, word );
-    return word;
+    return;
   }
   UnicodeString uWord = UTF8ToUnicode(word);
   uWord.toLower();
+  Classify( uWord );
+  filterTag( pos ); 
+  makeUnique(); 
+  getFoLiAResult( sword, uWord ); 
+}
+
+void Mblem::Classify( const UnicodeString& uWord ){
   mblemResult.clear();
   string inst = make_instance(uWord);  
   string classString;
@@ -263,89 +307,45 @@ string Mblem::Classify( Word *sword ){
     *Log(mblemLog) << "class: " << classString  << endl;
   // 1st find all alternatives
   vector<string> parts;
-  int numParts = TiCC::split_at( classString, parts, "|" );
+  int numParts = split_at( classString, parts, "|" );
   if ( numParts < 1 ){
     *Log(mblemLog) << "no alternatives found" << endl;
   }
   int index = 0;
   while ( index < numParts ) {
-    UnicodeString part = UTF8ToUnicode( parts[index++] );
-    if (debug)
-      *Log(mblemLog) <<"part = " << part << endl;
-    UnicodeString lemma = "";
+    string partS = parts[index++]; 
+    UnicodeString lemma;
     string restag;
-    int lpos = part.indexOf("+");
-    if ( lpos < 0 ){ // nothing to edit
-      restag = UnicodeToUTF8( part );
+    string::size_type pos = partS.find("+");
+    if ( pos == string::npos ){ 
+      // nothing to edit
+      restag = partS;
       lemma = uWord;
     }
     else {
-      restag = UnicodeToUTF8( UnicodeString( part, 0, lpos ) );
-      UnicodeString editstr = UnicodeString( part, lpos );
+      // some edit info available, like: WW(27)+Dgekomen+Ikomen
+      vector<string> edits;
+      size_t n = split_at( partS, edits, "+" );
+      if ( n < 1 )
+	throw runtime_error( "invalid editstring: " + partS );
+      restag = edits[0]; // the first one is the POS tag
+
       UnicodeString insstr;
       UnicodeString delstr;
       UnicodeString prefix;
-      bool done = false;
-      for ( int lpos=0; !done && lpos < editstr.length(); ++lpos ) {
-	if ( debug )
-	  *Log(mblemLog) << "editstr[" << lpos << "] = " 
-			 << UnicodeToUTF8(editstr[lpos]) << endl;
-	switch( editstr[lpos] ) {
-	case 'P': {
-	  if (editstr[lpos-1] =='+') {
-	    lpos++;
-	    int tmppos = editstr.indexOf("+", lpos);
-	    if ( debug )
-	      *Log(mblemLog) << "tmppos = " << tmppos << endl;
-	    if ( tmppos >=0 ){
-	      prefix = UnicodeString( editstr, lpos, tmppos - lpos );
-	      lpos = tmppos - 1;
-	    }
-	    else {
-	      prefix = UnicodeString( editstr, lpos );
-	      done = true;
-	    }
-	    if (debug)
-	      *Log(mblemLog) << "prefix=" << prefix << endl;
-	  }
+      for ( size_t l=1; l < edits.size(); ++l ) {
+	switch ( edits[l][0] ){
+	case 'P':
+	  prefix = UTF8ToUnicode( edits[l].substr( 1 ) );
 	  break;
-	}
-	case 'D': {
-	  if (editstr[lpos-1] =='+') {
-	    lpos++;
-	    int tmppos = editstr.indexOf("+", lpos);
-	    if ( tmppos >= 0 ){
-	      delstr = UnicodeString( editstr, lpos, tmppos - lpos );
-	      lpos = tmppos - 1;
-	    }
-	    else {
-	      delstr = UnicodeString( editstr, lpos );
-	      done = true;
-	    }
-	    if (debug)
-	      *Log(mblemLog) << "delstr=" << delstr << endl;
-	  }
+	case 'I':
+	  insstr = UTF8ToUnicode( edits[l].substr( 1 ) );
 	  break;
-	}
-	case 'I': {
-	  if (editstr[lpos-1] =='+') {
-	    lpos++;
-	    int tmppos = editstr.indexOf("+", lpos);
-	    if ( tmppos >=0 ){
-	      insstr = UnicodeString( editstr, lpos, tmppos - lpos);
-	      lpos = tmppos - 1;
-	    }
-	    else {
-	      insstr = UnicodeString( editstr, lpos);
-	      done = true;
-	    }
-	    if (debug)
-	      *Log(mblemLog) << "insstr=" << insstr << endl;
-	  }
+	case 'D':
+	  delstr =  UTF8ToUnicode( edits[l].substr( 1 ) );
 	  break;
-	}
 	default:
-	  break;
+	  *Log(mblemLog) << "Error: strange value in editstring: " << edits[l] << endl;
 	}
       }
       if (debug){
@@ -354,6 +354,9 @@ string Mblem::Classify( Word *sword ){
       }	
       int prefixpos = 0;
       if ( !prefix.isEmpty() ) {
+	// Whenever Toads makemblem is improved, (the infamous 
+	// 'tegemoetgekomen' example), this should probably
+	// become prefixpos = uWord.lastIndexOf(prefix);
 	prefixpos = uWord.indexOf(prefix);
 	if (debug)
 	  *Log(mblemLog) << "prefixpos = " << prefixpos << endl;
@@ -371,20 +374,24 @@ string Mblem::Classify( Word *sword ){
 	prefixpos = prefixpos + prefix.length();
       }
       if (debug)
-	*Log(mblemLog) << "post prefix != 0 word: "<< uWord 
+	*Log(mblemLog) << "post word: "<< uWord 
 		       << " lemma: " << lemma
 		       << " prefix: " << prefix
-		       << " insstr: " << insstr
 		       << " delstr: " << delstr
-		       << " l_delstr=" << delstr.length()
-		       << " l_word=" << uWord.length()
+		       << " insstr: " << insstr
 		       << endl;
       
       if ( uWord.endsWith( delstr ) ){
 	if ( uWord.length() > delstr.length() ){
 	  // chop delstr from the back, but do not delete the whole word
-	  UnicodeString part = UnicodeString( uWord, prefixpos, uWord.length() - delstr.length() - prefixpos );
-	  lemma += part + insstr;
+	  if ( uWord.length() == delstr.length() + 1  && insstr.isEmpty() ){
+	    // do not mangle short words too
+	    lemma += uWord;
+	  }
+	  else {
+	    UnicodeString part = UnicodeString( uWord, prefixpos, uWord.length() - delstr.length() - prefixpos );
+	    lemma += part + insstr;
+	  }
 	}
 	else if ( insstr.isEmpty() ){
 	  // no replacement, just take part after the prefix
@@ -424,6 +431,13 @@ string Mblem::Classify( Word *sword ){
 		     << "\ttag alt: " << mblemResult[index].getTag() << endl;
     }
   }
-  string res = postprocess( sword ); 
-  return res;
+}
+
+vector<pair<string,string> > Mblem::getResult() const {
+  vector<pair<string,string> > result;
+  for ( size_t i=0; i < mblemResult.size(); ++i ){
+    result.push_back( make_pair( mblemResult[i].getLemma(),
+				 mblemResult[i].getTag() ) );
+  }
+  return result;
 }
