@@ -1,8 +1,8 @@
 /*
-  $Id: mbma_mod.cxx 14501 2012-03-22 13:00:16Z sloot $
+  $Id: mbma_mod.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mbma_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -34,9 +34,7 @@
 #include <timbl/TimblAPI.h>
 
 #include "frog/Frog.h"
-#include "frog/Configuration.h"
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
+#include "ticcutils/Configuration.h"
 #include "frog/mbma_mod.h"
 
 using namespace std;
@@ -46,7 +44,7 @@ using namespace TiCC;
 const long int LEFT =  6; // left context
 const long int RIGHT = 6; // right context
 
-Mbma::Mbma(): MTreeFilename( "dm.igtree" ), MTree(0) { 
+Mbma::Mbma(): MTreeFilename( "dm.igtree" ), MTree(0), transliterator(0) { 
   mbmaLog = new LogStream( theErrLog, "mbma-" );
 }
 
@@ -107,7 +105,7 @@ void Mbma::init_cgn( const string& dir ) {
   if ( tc ){
     while( getline( tc, line) ) {
       vector<string> tmp;
-      size_t num = TiCC::split_at(line, tmp, " ");
+      size_t num = split_at(line, tmp, " ");
       if ( num < 2 ){
 	*Log(mbmaLog) << "splitting '" << line << "' failed" << endl;
 	throw ( runtime_error("panic") );
@@ -122,7 +120,7 @@ void Mbma::init_cgn( const string& dir ) {
   if ( tc1 ){
     while( getline(tc1, line) ) {
       vector<string> tmp;
-      size_t num = TiCC::split_at(line, tmp, " ");
+      size_t num = split_at(line, tmp, " ");
       if ( num == 2 )
 	TAGconv.insert( make_pair( tmp[0], tmp[1] ) ); 
     }
@@ -130,13 +128,24 @@ void Mbma::init_cgn( const string& dir ) {
   else
     throw ( runtime_error( "unable to open:" + fn ) );
 }
+
+void Mbma::init_filter( ){
+  UErrorCode stat = U_ZERO_ERROR;
+  transliterator = Transliterator::createInstance( "NFD; [:M:] Remove; NFC",
+						   UTRANS_FORWARD,
+						   stat );
+  if ( U_FAILURE( stat ) ){
+    throw runtime_error( "initFilter FAILED !" );
+  }
+}
+
   
 bool Mbma::init( const Configuration& config ) {
   *Log(mbmaLog) << "Initiating morphological analyzer..." << endl;
   debugFlag = tpDebug;
   string db = config.lookUp( "debug", "mbma" );
   if ( !db.empty() )
-    debugFlag = stringTo<int>( db );
+    debugFlag = TiCC::stringTo<int>( db );
   string val = config.lookUp( "version", "mbma" );
   if ( val.empty() ){
     version = "1.0";
@@ -149,13 +158,25 @@ bool Mbma::init( const Configuration& config ) {
   }
   else
     tagset = val;
+
+  val = config.lookUp( "set", "tagger" );
+  if ( val.empty() ){
+    cgn_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
+  }
+  else
+    cgn_tagset = val;
+
   string tfName = config.lookUp( "treeFile", "mbma" );
   if ( tfName.empty() )
     tfName = "mbma.igtree";
   MTreeFilename = prefix( config.configDir(), tfName );
   fillMaps();
   init_cgn( config.configDir() );
-    //Read in (igtree) data
+  string dof = config.lookUp( "filter_diacritics", "mbma" );
+  if ( dof == "true" || dof == "TRUE" || dof =="yes" || dof == "YES" ){
+    init_filter();
+  }
+  //Read in (igtree) data
   string opts = config.lookUp( "timblOpts", "mbma" );
   if ( opts.empty() )
     opts = "-a1";
@@ -204,33 +225,25 @@ vector<string> Mbma::make_instances( const UnicodeString& word ){
 }
   
 string extract( const string& line, size_t start, char to ){
-  return line.substr( start, line.find( to , start ) - start );
+  string::size_type pos = line.find( to , start );
+  if ( to == '>' && pos == string::npos ){
+    cerr << "invalid edit value in line " << line << endl;
+    return "";
+  }
+  return line.substr( start, pos - start );
 }
 
-string find_class( unsigned int step, 
-		   const vector<string>& classes,
-		   unsigned int nranal ){
-  string result = classes[0];
-  if ( nranal > 1 ){
-    if ( classes.size() > 1 ){
-      if ( classes.size() >= step )
-	result = classes[step-1];
-      else
-	result = "0";
-    }
-  }
-  return result;
-}
-  
 string Mbma::calculate_ins_del( const string& in_class, 
 				string& deletestring,
-				string& insertstring ){
+				string& insertstring,
+				bool& participle ){
   string result_class = in_class;
   size_t pos = in_class.find("+");
   if ( pos != string::npos ) { 
     if ( debugFlag){
       *Log(mbmaLog) << "calculate ins/del for " << in_class << endl;
     }
+    participle = ( in_class.find( 'p' ) < pos );
     pos++;
     if (in_class[pos]=='D') { // delete operation 
       deletestring = extract( in_class, pos+1, '/' );
@@ -256,6 +269,7 @@ string Mbma::calculate_ins_del( const string& in_class,
   else {
     if ( debugFlag ){
       *Log(mbmaLog) << "calculate_ins_del: do nothing" << endl;
+      *Log(mbmaLog) << "result class=" << result_class << endl;
     }
   }
   return result_class;
@@ -274,7 +288,7 @@ ostream& operator<< ( ostream& os, const vector<waStruct>& V ){
 }
 
 vector<waStruct> Mbma::Step1( unsigned int step, 
-			      const UnicodeString& word, int nranal,
+			      const UnicodeString& word,
 			      const vector<vector<string> >& classParts,
 			      const string& basictags ) {
   vector<waStruct> ana;
@@ -283,13 +297,14 @@ vector<waStruct> Mbma::Step1( unsigned int step,
   string previoustag;
   waStruct waItem;
   for ( long k=0; k < word.length(); ++k ) { 
-    this_class = find_class( step, classParts[k], nranal );
+    this_class = classParts[step][k];
     if ( debugFlag){
       *Log(mbmaLog) << "Step::" << step << " " << this_class << endl;
     }
     string deletestring;
     string insertstring;
-    this_class = calculate_ins_del( this_class, deletestring, insertstring);
+    bool participle = false;
+    this_class = calculate_ins_del( this_class, deletestring, insertstring, participle );
     if ( deletestring == "eeer" )
       deletestring = "eer";
     /* exceptions */
@@ -299,16 +314,16 @@ vector<waStruct> Mbma::Step1( unsigned int step,
       eexcept = true;
     }
     // insert the deletestring :-) 
-    UnicodeString last = UTF8ToUnicode( deletestring );
-    waItem.word += last;
+    UnicodeString dels = UTF8ToUnicode( deletestring );
+    waItem.word += dels;
     // delete the insertstring :-) 
-    if (( tobeignored == 0 ) &&
-	( insertstring != "ge" ) &&
-	( insertstring != "be" ) )
+    if ( tobeignored == 0 &&
+	 ( !participle || 
+	   ( insertstring != "ge" &&
+	     insertstring != "be" ) ) )
       tobeignored = insertstring.length();
     
-    if ( basictags.find(this_class[0]) != string::npos &&
-	 this_class != "PE" ) { 
+    if ( basictags.find(this_class[0]) != string::npos ){
       // encountering POS tag
       if ( !previoustag.empty() ) { 
 	waItem.act = previoustag;
@@ -319,6 +334,7 @@ vector<waStruct> Mbma::Step1( unsigned int step,
     }
     else { 
       if ( this_class[0] !='0' ){ 
+	// encountering inflection info
 	if ( !previoustag.empty() ) { 
 	  waItem.act = previoustag;
 	  ana.push_back( waItem );
@@ -345,6 +361,7 @@ vector<waStruct> Mbma::Step1( unsigned int step,
   // do we have inflection?
   size_t pos = this_class.find( '/' );
   if ( pos != string::npos && this_class != "E/P" ) { 
+    // tags like 0/te3 N/e 0/P etc.
     string inflection = "i" + this_class.substr( pos+1 );
     waItem.act = inflection;
     ana.push_back( waItem );
@@ -352,7 +369,7 @@ vector<waStruct> Mbma::Step1( unsigned int step,
   return ana;
 }
 
-string change_tag( const char ch ){
+string select_tag( const char ch ){
   string newtag;
   switch( ch ){
   case 'm':
@@ -385,34 +402,36 @@ string change_tag( const char ch ){
 void Mbma::resolve_inflections( vector<waStruct>& ana, 
 				const string& basictags ) {
   // resolve all clearly resolvable implicit selections of inflectional tags
-  vector<waStruct>::iterator prev = ana.begin();
-  vector<waStruct>::const_iterator it = ana.begin();
-  int step = 0;
-  while ( it != ana.end() ){
-    string this_class = it->act;
-    if ( step > 1 ){
-      ++prev;
-    }
-    ++it;
-    ++step;
-    if ( this_class[0]=='i') { 
+  for ( vector<waStruct>::iterator it = ana.begin(); 
+	it != ana.end(); 
+	++it ){
+    if ( ana.begin() == it )
+      continue; // start at second
+    string act = it->act;
+    if ( act[0]=='i') { 
+      // it is an inflection tag
       if (debugFlag){
-	*Log(mbmaLog) << "thisclass >" << this_class << "<" << endl;
+	*Log(mbmaLog) << "act: >" << act << "<" << endl;
       }	  
-      /* given the specific selections of certain inflections,
-	 change the tag!  */
-      string newtag = change_tag( this_class[1] );
+      // given the specific selections of certain inflections,
+      //    select a tag!
+      string new_tag = select_tag( act[1] );
       
-      /* apply the change. Remember, the idea is that an inflection is
-	 far more certain of the tag of its predecessing morpheme than
-	 the morpheme itself. This is not always the case, but it works  */
-      if ( !newtag.empty() ) {
+      // apply the change. Remember, the idea is that an inflection is
+      // far more certain of the tag of its predecessing morpheme than
+      // the morpheme itself. 
+      // This is not always the case, but it works
+      if ( !new_tag.empty() ) {
 	if ( debugFlag  ){
-	  *Log(mbmaLog) << "selects " << newtag << endl;
+	  *Log(mbmaLog) << act[1] << " selects " << new_tag << endl;
 	}
-	/* go left */
-	if ( basictags.find(prev->act[0]) != string::npos ){
-	  prev->act[0] = newtag[0];
+	// change the previous act
+	if ( basictags.find((it-1)->act[0]) != string::npos ){
+	  if ( debugFlag  ){
+	    *Log(mbmaLog) << "replace " << (it-1)->act[0] << " by " 
+			  << new_tag[0] << endl;
+	  }
+	  (it-1)->act[0] = new_tag[0];
 	}
       }
     }
@@ -422,17 +441,20 @@ void Mbma::resolve_inflections( vector<waStruct>& ana,
 MBMAana Mbma::addInflect( const vector<waStruct>& ana,
 			  const string& inflect, 
 			  const vector<string>& morph ){
-  bool found = false;
   string the_act;
   vector<waStruct>::const_reverse_iterator it = ana.rbegin();
-  while ( !found && it != ana.rend() ) { 
+  while ( it != ana.rend() ) { 
     // go back to the last non-inflectional tag 
     the_act = it->act;
     if (debugFlag){
-      *Log(mbmaLog) << "final tag " << the_act << endl;
+      *Log(mbmaLog) << "examine act " << the_act << endl;
+    }
+    if ( the_act[0] != 'i' ){
+      if (debugFlag){
+	*Log(mbmaLog) << "final tag " << the_act << endl;
+      }
+      break;
     }
-    if ( the_act[0] != 'i' )
-      found=true;
     else { 
       ++it;
     }
@@ -447,17 +469,15 @@ MBMAana Mbma::addInflect( const vector<waStruct>& ana,
   map<string,string>::const_iterator tit = tagNames.find( tag ); 
   if ( tit == tagNames.end() ){
     // unknown tag
-    if (debugFlag)
-      *Log(mbmaLog) << "added X 4" << endl;
     outTag = "X";
     descr = "unknown";
   }
   else {
-    if (debugFlag)
-      *Log(mbmaLog) << "added (4) " << tit->second << endl;
     outTag = tit->first;
     descr = tit->second;
   }
+  if (debugFlag)
+    *Log(mbmaLog) << "added (4) " << outTag << " " << descr << endl;
   return MBMAana( outTag, inflect, morph, descr );
 }
 
@@ -500,26 +520,33 @@ MBMAana Mbma::inflectAndAffix( const vector<waStruct>& ana ){
   }
   if (debugFlag)	
     *Log(mbmaLog) << "inflectAndAffix: " << inflect << " - " << morphemes << endl;
-  if ( inflect.empty() ){
-    if (debugFlag)
-      *Log(mbmaLog) << "Inflection: none " << endl;
-    MBMAana retVal;
-    return retVal;
-  }
-  else {
-    // go back to the last non-inflectional
-    // tag and stick it at the end
-    MBMAana retVal = addInflect( ana, inflect, morphemes );
-    if (debugFlag)
-      *Log(mbmaLog) << "Inflection: " << retVal << endl;
-    return retVal;
-  }
+  // go back to the last non-inflectional
+  // tag and stick it at the end. 
+  MBMAana retVal = addInflect( ana, inflect, morphemes );
+  if (debugFlag)
+    *Log(mbmaLog) << "Inflection: " << retVal << endl;
+  return retVal;
 }
 
-void Mbma::execute( const UnicodeString& word, 
-		    const vector<string>& classes ){
-  const string basictags = "NAQVDOBPYIXZ";
-  analysis.clear();
+#define OLD_STEP
+
+#ifdef OLD_STEP
+string find_class( unsigned int step, 
+		   const vector<string>& classes,
+		   unsigned int nranal ){
+  string result = classes[0];
+  if ( nranal > 1 ){
+    if ( classes.size() > 1 ){
+      if ( classes.size() > step )
+	result = classes[step];
+      else
+	result = "0";
+    }
+  }
+  return result;
+}
+
+vector<vector<string> > generate_all_perms( const vector<string>& classes ){
   // determine all alternative analyses, remember the largest
   // and store every part in a vector of string vectors
   int largest_anal=1;
@@ -527,7 +554,7 @@ void Mbma::execute( const UnicodeString& word,
   classParts.resize( classes.size() );
   for ( unsigned int j=0; j< classes.size(); ++j ){
     vector<string> parts;
-    int num = TiCC::split_at( classes[j], parts, "|" );
+    int num = split_at( classes[j], parts, "|" );
     if ( num > 0 ){
       classParts[j] = parts;
       if ( num > largest_anal )
@@ -540,19 +567,92 @@ void Mbma::execute( const UnicodeString& word,
       classParts[j] = dummy;
     }
   }
+  //
+  // now expand the result
+  vector<vector<string> > result;
+  for ( int step=0; step < largest_anal; ++step ){
+    vector<string> item(classParts.size());
+    for ( size_t k=0; k < classParts.size(); ++k ) { 
+      item[k] = find_class( step, classParts[k], largest_anal );
+    }
+    result.push_back( item );
+  }
+  return result;
+}
+#else  
+
+bool next_perm( vector< vector<string>::const_iterator >& its,
+		const vector<vector<string> >& parts ){
+  for ( size_t i=0; i < parts.size(); ++i ){
+    ++its[i];
+    if ( its[i] == parts[i].end() ){
+      if ( i == parts.size() -1 )
+	return false;
+      its[i] = parts[i].begin();
+    }
+    else 
+      return true;
+  }
+  return false;
+}
+
+vector<vector<string> > generate_all_perms( const vector<string>& classes ){
   
-  if (debugFlag){
+  // determine all alternative analyses
+  // store every part in a vector of string vectors
+  vector<vector<string> > classParts;
+  classParts.resize( classes.size() );
+  for ( unsigned int j=0; j< classes.size(); ++j ){
+    vector<string> parts;
+    int num = split_at( classes[j], parts, "|" );
+    if ( num > 0 ){
+      classParts[j] = parts;
+    }
+    else {
+      // only one, create a dummy
+      vector<string> dummy;
+      dummy.push_back( classes[j] );
+      classParts[j] = dummy;
+    }
+  }
+  //
+  // now expand
+  vector< vector<string>::const_iterator > its( classParts.size() );
+  for ( size_t i=0; i<classParts.size(); ++i ){
+    its[i] = classParts[i].begin();
+  }
+  vector<vector<string> > result;
+  bool more = true;
+  while ( more ){
+    vector<string> item(classParts.size());
+    for( size_t j=0; j< classParts.size(); ++j ){
+      item[j] = *its[j];
+    }
+    result.push_back( item );
+    more = next_perm( its, classParts );
+  }
+  return result;
+}
+#endif
+
+void Mbma::execute( const UnicodeString& word, 
+		    const vector<string>& classes ){
+  const string basictags = "NAQVDOBPYIXZ";
+  analysis.clear();
+
+  vector<vector<string> > allParts = generate_all_perms( classes );
+  if ( debugFlag ){
     string out = "alternatives: word=" + UnicodeToUTF8(word) + ", classes=<";
     for ( size_t i=0; i < classes.size(); ++i )
       out += classes[i] + ",";
     out += ">";
     *Log(mbmaLog) << out << endl;
+    *Log(mbmaLog) << "allParts : " << allParts << endl;
   }    
   
   // now loop over all the analysis
-  for ( unsigned int step=largest_anal; step> 0; --step ) { 
-    vector<waStruct> ana = Step1( step, word, largest_anal, 
-				  classParts, basictags );
+  for ( unsigned int step=0; step < allParts.size(); ++step ) { 
+    vector<waStruct> ana = Step1( step, word, allParts, basictags );
     if (debugFlag)
       *Log(mbmaLog) << "intermediate analysis 1: " << ana << endl;
     resolve_inflections( ana, basictags );
@@ -564,13 +664,30 @@ void Mbma::execute( const UnicodeString& word,
   }
 }
 
+void Mbma::addAltMorph( Word *word, 
+			const vector<string>& lemmas ) const {
+  Alternative *alt = new Alternative();
+  MorphologyLayer *ml = new MorphologyLayer();
+#pragma omp critical(foliaupdate)
+  {
+    alt->append( ml );
+    word->append( alt );
+  }
+  addMorph( ml, lemmas );
+}
+
 void Mbma::addMorph( Word *word, 
-		     const vector<string>& lemmas ){
+		     const vector<string>& lemmas ) const {
   MorphologyLayer *ml = new MorphologyLayer();
 #pragma omp critical(foliaupdate)
   {
     word->append( ml );
   }
+  addMorph( ml, lemmas );
+}
+
+void Mbma::addMorph( MorphologyLayer *ml, 
+		     const vector<string>& lemmas ) const {
   int offset = 0;
   for ( size_t p=0; p < lemmas.size(); ++p ){
     Morpheme *m = new Morpheme();
@@ -589,170 +706,217 @@ void Mbma::addMorph( Word *word,
     }
   }
 }	  
-      
-void Mbma::postprocess( Word *fword, PosAnnotation *pos ){
+
+void Mbma::filterTag( const string& head,  const vector<string>& feats ){
+  // first we select only the matching heads
   if (debugFlag){
+    *Log(mbmaLog) << "filter with tag, head: " << head 
+		  << " feats: " << feats << endl;
+    *Log(mbmaLog) << "filter:start, analysis is:" << endl;
+    int i=1;
     for(vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
-      *Log(mbmaLog) << "tag " <<  it->getTag() + " inflection: " << it->getInflection() << endl;
-    *Log(mbmaLog) << "morph analyses: " << endl;;
-    for (vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
-      *Log(mbmaLog) << it->getMorph() << endl;
-    *Log(mbmaLog) << "before morpho: " << endl;
+      *Log(mbmaLog) << i++ << " - " << *it << endl;
   }
-  const string tag = pos->feat("head");
-  map<string,string>::const_iterator tagIt = TAGconv.find( tag );
+  map<string,string>::const_iterator tagIt = TAGconv.find( head );
   if ( tagIt == TAGconv.end() ) {
-    //
     // this should never happen
-    //
-    if (debugFlag){
-      *Log(mbmaLog) << "no match!" << endl;
-    } 
-    throw ValueError( "unknown pos tag value '" + tag + "'" );
+    throw ValueError( "unknown head feature '" + head + "'" );
   }
-  else {
-    if (debugFlag){
-      *Log(mbmaLog) << "#matches: " << tagIt->first << " " << tagIt->second << endl;
-    }
-    size_t match = 0;	
-    vector<size_t> matches;
-    for ( size_t m_index = 0; m_index <analysis.size(); ++m_index ) {
+  if (debugFlag){
+    *Log(mbmaLog) << "#matches: " << tagIt->first << " " << tagIt->second << endl;
+  }
+  vector<MBMAana>::iterator ait = analysis.begin();
+  while ( ait != analysis.end() ){
+    if ( tagIt->second == ait->getTag() ) {
       if (debugFlag)
-	*Log(mbmaLog) << "comparing " << tagIt->second << " with " << analysis[m_index].getTag() << endl;
-      if ( tagIt->second == analysis[m_index].getTag() ) {
-	match++;
-	matches.push_back( m_index );
-      }
+	*Log(mbmaLog) << "comparing " << tagIt->second << " with " 
+		      << ait->getTag() << " (OK)" << endl;
+      ait++;
     }
-    if (debugFlag) {
-      *Log(mbmaLog) << "main tag " << tagIt->second
-		    << " matches " << match 
-		    << " morpho analyses: " << endl;
-      for( size_t p=0; p < match; ++p )  {
-	*Log(mbmaLog) << "\tmatch #" << p << " : " << analysis[matches[p]].getMorph() << endl;
-      }
-    }
-    //should be(come?) a switch
-    if ( match == 0 ) {
-      // fallback option: use the word and pretend it's a lemma ;-)
-      UnicodeString word;
-#pragma omp critical(foliaupdate)
-      {
-	word = fword->text();
-      }
-      word.toLower();
-      vector<string> tmp;
-      tmp.push_back( UnicodeToUTF8(word) );
-      addMorph( fword, tmp );
-    }
-    else if (match == 1) {
-      const vector<string> ma = analysis[matches[0]].getMorph();
-      addMorph( fword, ma );
-    } 
     else {
       if (debugFlag)
-	*Log(mbmaLog) << "multiple lemma's" << endl;
-      map<string, int> possible_lemmas;
-      // find unique lemma's
-      for ( size_t q = 0; q < match; ++q ) {
-	const vector<string> ma = analysis[matches[q]].getMorph();
-	string key;
-	for ( size_t p=0; p < ma.size(); ++p )
-	  key += ma[p];
-	map<string, int>::iterator lemmaIt = possible_lemmas.find( key );
-	if (lemmaIt == possible_lemmas.end()){
-	  possible_lemmas.insert( make_pair( key,1 ) );
-	}
-	else {
-	  lemmaIt->second++;
-	}
-      }
-      if (debugFlag)
-	*Log(mbmaLog) << "#unique lemma's: " << possible_lemmas.size() << endl;
-      if ( possible_lemmas.size() < 1) {
-	if (debugFlag)
-	  *Log(mbmaLog) << "Problem!, no possible lemma's" << endl;
-	return;
-      }
-      // append all unique lemmas
-      // find best match
-      // loop through all subParts of the tag
-      // and match with inflections from each m
-      vector<Feature*> feats = pos->select<Feature>();
-      if (debugFlag){
-	*Log(mbmaLog) << "tag: " << tag << endl;
-	for ( size_t q =0 ; q < feats.size(); ++q ) {
-	  *Log(mbmaLog) << "\tpart #" << q << ": " << feats[q]->cls() << endl;
-	}
-      }
-      map <string, const vector<string>* > bestMatches;
-      int max_count = 0;
-      for ( size_t q=0; q < match; ++q ) {
-	int match_count = 0;
-	string inflection = analysis[matches[q]].getInflection();
-	if (debugFlag)
-	  *Log(mbmaLog) << "matching " << inflection << " with " << tag << endl;
-	for ( size_t u=0; u < feats.size(); ++u ) {
-	  map<string,string>::const_iterator conv_tag_p = TAGconv.find(feats[u]->cls());
-	  if (conv_tag_p != TAGconv.end()) {
-	    string c = conv_tag_p->second;
-	    if ( inflection.find( c ) != string::npos )
-	      match_count++;
-	  }
-	}
-	if (debugFlag)
-	  *Log(mbmaLog) << "score: " << match_count << " max was " << max_count << endl;
-	if (match_count >= max_count) {
-	  string key;
-	  const vector<string> *ma = &analysis[matches[q]].getMorph();
-	  for ( size_t p=0; p < ma->size(); ++p )
-	    key += (*ma)[p] + "+"; // create uniqe keys
-	  
-	  if (match_count > max_count) {
-	    max_count = match_count;
-	    bestMatches.clear();
-	  }
-	  bestMatches.insert( make_pair(key, ma ) );
-	}
-      }
-      map< string, const vector<string> *>::const_iterator it = bestMatches.begin();
-      while ( it != bestMatches.end() ){
-	if (debugFlag){
-	  string tmp;
-	  for ( size_t p=0; p < it->second->size(); ++p )
-	    tmp += "[" + (*it->second)[p] + "]";
-	  *Log(mbmaLog) << "MBMA add lemma: " << tmp << endl;
-	}
-	addMorph( fword, *it->second );
-	++it;
-      }
+	*Log(mbmaLog) << "comparing " << tagIt->second << " with " 
+		      << ait->getTag() << " (rejected)" << endl;
+      ait = analysis.erase( ait );
     }
   }
-}  // postprocess
+  if (debugFlag){
+    *Log(mbmaLog) << "filter: analysis after first step" << endl;
+    int i=1;
+    for(vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
+      *Log(mbmaLog) << i++ << " - " << *it << endl;
+  }
 
+  if ( analysis.size() < 1 ){
+    if (debugFlag ){
+      *Log(mbmaLog) << "analysis has size: " << analysis.size() 
+		    << " so skip next filter" << endl;
+    }
+    return;
+  }
+  // ok there are several analysis left.
+  // try to select on the features:
+  //
+  // find best match
+  // loop through all subfeatures of the tag
+  // and match with inflections from each m
+  set<const MBMAana *> bestMatches;
+  int max_count = 0;
+  for ( size_t q=0; q < analysis.size(); ++q ) {
+    int match_count = 0;
+    string inflection = analysis[q].getInflection();
+    if (debugFlag){
+      *Log(mbmaLog) << "matching " << inflection << " with " << feats << endl;
+    }
+    for ( size_t u=0; u < feats.size(); ++u ) {
+      map<string,string>::const_iterator conv_tag_p = TAGconv.find(feats[u]);
+      if (conv_tag_p != TAGconv.end()) {
+	string c = conv_tag_p->second;
+	if (debugFlag){
+	  *Log(mbmaLog) << "found " << feats[u] << " ==> " << c << endl;
+	}
+	if ( inflection.find( c ) != string::npos ){
+	  if (debugFlag){
+	    *Log(mbmaLog) << "it is in the inflection " << endl;
+	  }
+	  match_count++;
+	}
+      }
+    }
+    if (debugFlag)
+      *Log(mbmaLog) << "score: " << match_count << " max was " << max_count << endl;
+    if (match_count >= max_count) {
+      if (match_count > max_count) {
+	max_count = match_count;
+	bestMatches.clear();
+      }
+      bestMatches.insert(&analysis[q]);
+    }
+  }
+  // so now we have "the" best matches.
+  // Weed the rest
+  vector<const MBMAana*> res;
+  set<const MBMAana*>::const_iterator bit = bestMatches.begin();
+  while ( bit != bestMatches.end() ){
+    res.push_back( *bit );
+    ++bit;
+  }
+  if (debugFlag){
+    *Log(mbmaLog) << "filter: analysis after second step" << endl;
+    int i=1;
+    for(vector<const MBMAana*>::const_iterator it=res.begin(); it != res.end(); it++)
+      *Log(mbmaLog) << i++ << " - " <<  *it << endl;
+    *Log(mbmaLog) << "start looking for doubles" << endl;
+  }
+  //
+  // but now we still might have doubles
+  //
+  map<string, const MBMAana*> unique;
+  vector<const MBMAana*>::iterator it=res.begin();
+  while (  it != res.end() ){
+    vector<string> v = (*it)->getMorph();
+    string tmp;
+    // create an unique key
+    for ( size_t p=0; p < v.size(); ++p ) {
+      tmp += v[p] + "+";
+    }
+    unique[tmp] = *it;
+    ++it;
+  }
+  vector<MBMAana> result;
+  map<string, const MBMAana*>::const_iterator uit=unique.begin();
+  while ( uit != unique.end() ){
+    result.push_back( *(uit->second) );
+    ++uit;
+  }
+  analysis = result;
+  if (debugFlag){
+    *Log(mbmaLog) << "filter: definitive analysis:" << endl;
+    int i=1;
+    for(vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
+      *Log(mbmaLog) << i++ << " - " << *it << endl;
+    *Log(mbmaLog) << "done filtering" << endl;
+  }
+  return;
+}
+
+void Mbma::getFoLiAResult( Word *fword, const UnicodeString& uword ) const {
+  if ( analysis.size() == 0 ){
+    // fallback option: use the lowercased word and pretend it's a lemma ;-)
+    UnicodeString word = uword;
+    word.toLower();
+    if ( debugFlag ){
+      *Log(mbmaLog) << "no matches found, use the word instead: " 
+		    << word << endl;
+    }
+    vector<string> tmp;
+    tmp.push_back( UnicodeToUTF8(word) );
+    addMorph( fword, tmp );
+  }
+  else {
+    vector<MBMAana>::const_iterator sit = analysis.begin();
+    while( sit != analysis.end() ){
+      if ( sit == analysis.begin() )
+	addMorph( fword, sit->getMorph() );
+      else
+	addAltMorph( fword, sit->getMorph() );
+      ++sit;
+    }
+  }
+}
+  
 void Mbma::addDeclaration( Document& doc ) const {
   doc.declare( AnnotationType::MORPHOLOGICAL, tagset,
 	       "annotator='frog-mbma-" +  version + 
-	       "', annotatortype='auto'");
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
-  
-bool Mbma::Classify( Word* sword ){
+
+UnicodeString Mbma::filterDiacritics( const UnicodeString& in ) const {
+  if ( transliterator ){
+    UnicodeString result = in;
+    transliterator->transliterate( result );
+    return result;
+  }
+  else
+    return in;
+}
+
+void Mbma::Classify( Word* sword ){
+  if ( sword->isinstance(PlaceHolder_t) )
+    return;
   UnicodeString uWord;
   PosAnnotation *pos;
+  string head;
 #pragma omp critical(foliaupdate)
   {
     uWord = sword->text();
-    pos = sword->annotation<PosAnnotation>();
+    pos = sword->annotation<PosAnnotation>( cgn_tagset );
+    head = pos->feat("head");
   }
-  string tag = pos->feat("head");
-  if ( tag == "SPEC" || tag == "LET" ){
+  if ( head == "SPEC" || head == "LET" ){
     string word = UnicodeToUTF8( uWord );
     vector<string> tmp;
     tmp.push_back( word );
     addMorph( sword, tmp );
-    return true;
   }
+  else {
+    vector<string> featVals;
+#pragma omp critical(foliaupdate)
+    {
+      vector<folia::Feature*> feats = pos->select<folia::Feature>();
+      for ( size_t i = 0; i < feats.size(); ++i )
+	featVals.push_back( feats[i]->cls() );
+    }
+    Classify( uWord );
+    filterTag( head, featVals );
+    getFoLiAResult( sword, uWord );
+  }
+}
+
+void Mbma::Classify( const UnicodeString& word ){
+  UnicodeString uWord = word;
   uWord.toLower();
+  uWord = filterDiacritics( uWord );
   if (debugFlag)
     *Log(mbmaLog) << "Classify word: " << uWord << endl;
   
@@ -761,17 +925,34 @@ bool Mbma::Classify( Word* sword ){
   for( size_t i=0; i < insts.size(); ++i ) {
     string ans;
     MTree->Classify( insts[i], ans );
-    if ( debugFlag )
-      *Log(mbmaLog) << "itt #" << i << ": timbl gave class= " << ans << endl; 
+    if ( debugFlag ){
+      *Log(mbmaLog) << "itt #" << i << ": timbl gave class= " << ans 
+		    << ", it matched at depth=" << MTree->matchDepth() << endl; 
+    }
     classes.push_back( ans);
   }
-
+  
   // fix for 1st char class ==0
   if ( classes[0] == "0" )
     classes[0] = "X";  
   execute( uWord, classes );
-  postprocess( sword, pos );
-  return true;
+}
+
+vector<vector<string> > Mbma::getResult() const {
+  vector<vector<string> > result;
+  for (vector<MBMAana>::const_iterator it=analysis.begin(); 
+       it != analysis.end(); 
+       it++ ){
+    vector<string> mors = it->getMorph();
+    if ( debugFlag ){
+      *Log(mbmaLog) << "Morphs " << mors << endl;
+    }
+    result.push_back( mors );
+  }
+  if ( debugFlag ){
+    *Log(mbmaLog) << "result of morph analyses: " << result << endl;
+  }
+  return result;
 }
 
 ostream& operator<< ( ostream& os, const MBMAana& a ){
@@ -780,4 +961,12 @@ ostream& operator<< ( ostream& os, const MBMAana& a ){
   return os;
 }
 
+ostream& operator<< ( ostream& os, const MBMAana *a ){
+  if ( a )
+    os << *a;
+  else 
+    os << "no-mbma";
+  return os;
+}
+
 
