@@ -1,8 +1,8 @@
 /*
-  $Id: mwu_chunker_mod.cxx 14478 2012-03-19 16:10:38Z sloot $
+  $Id: mwu_chunker_mod.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mwu_chunker_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   This file is part of frog.
@@ -37,13 +37,11 @@
 #include "timbl/TimblAPI.h"
 
 #include "frog/Frog.h" // defines etc.
-#include "libfolia/folia.h" 
-#include "libfolia/document.h"
 #include "frog/mwu_chunker_mod.h"
 
-using namespace folia;
 using namespace std;
 using namespace TiCC;
+using namespace folia;
 
 mwuAna::mwuAna( Word *fwrd ){
   spec = false;
@@ -56,7 +54,7 @@ mwuAna::mwuAna( Word *fwrd ){
   if ( tag == "SPEC" ){
 #pragma omp critical(foliaupdate)
     {
-      vector<Feature*> feats = fwrd->select<Feature>();
+      vector<folia::Feature*> feats = fwrd->select<folia::Feature>( );
       spec = ( feats.size() == 1 && feats[0]->cls() == "deeleigen" );
     }
   }
@@ -126,8 +124,8 @@ bool Mwu::read_mwus( const string& fname) {
   string line;
   while( getline( mwufile, line ) ) {
     vector<string> res1, res2; //res1 has mwus and tags, res2 has ind. words
-    if ( ( TiCC::split_at(line, res1, " ") == 2 ) &&
-	 ( TiCC::split_at(res1[0], res2, "_") >= 2 ) ){
+    if ( ( split_at(line, res1, " ") == 2 ) &&
+	 ( split_at(res1[0], res2, "_") >= 2 ) ){
       string key = res2[0];
       res2.erase(res2.begin());
       MWUs.insert( make_pair( key, res2 ) );
@@ -145,7 +143,7 @@ bool Mwu::init( const Configuration& config ) {
   debug = tpDebug;
   string val = config.lookUp( "debug", "mwu" );
   if ( !val.empty() )
-    debug = stringTo<int>( val );
+    debug = TiCC::stringTo<int>( val );
   val = config.lookUp( "t", "mwu" );  
   if ( val.empty() ){
     *Log(mwuLog) << "cannot find attribute 't' in configfile" << endl;
@@ -168,6 +166,7 @@ bool Mwu::init( const Configuration& config ) {
   }
   else
     tagset = val;
+
   return true;
 }
 
@@ -182,20 +181,23 @@ void Mwu::addDeclaration( Document& doc ) const {
   doc.declare( AnnotationType::ENTITY, 
 	       tagset,
 	       "annotator='frog-mwu-" + version
-	       + "', annotatortype='auto'");
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
 
-void Mwu::Classify( Sentence *sent ){
+void Mwu::Classify( const vector<Word*>& words ){
+  if ( words.empty() )
+    return;
   reset();
-  vector<Word*> words;
-#pragma omp critical(foliaupdate)
-  {
-    words = sent->words();
+  for ( size_t i=0; i < words.size(); ++i ){
+    add( words[i] );
   }
-  for ( size_t i=0; i < words.size(); ++i )
-    add( words[i] );  
   Classify();
   EntitiesLayer *el = 0;
+  Sentence *sent;
+#pragma omp critical(foliaupdate)
+  {
+    sent = words[0]->sentence();
+  }
   for( size_t i = 0; i < mWords.size(); ++i ){
     el = mWords[i]->addEntity( tagset, sent, el );
   }
