@@ -1,8 +1,8 @@
 /*
-  $Id: ner_tagger_mod.cxx 14481 2012-03-21 11:26:55Z sloot $
+  $Id: ner_tagger_mod.cxx 15912 2013-04-03 14:04:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/ner_tagger_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -29,8 +29,6 @@
 */
 
 #include "mbt/MbtAPI.h"
-#include "libfolia/folia.h"
-#include "libfolia/document.h"
 #include "frog/Frog.h"
 #include "frog/ner_tagger_mod.h"
 
@@ -51,7 +49,7 @@ bool NERTagger::init( const Configuration& conf ){
   debug = tpDebug;
   string db = conf.lookUp( "debug", "NER" );
   if ( !db.empty() )
-    debug = folia::stringTo<int>( db );
+    debug = TiCC::stringTo<int>( db );
   switch ( debug ){
   case 0:
   case 1:
@@ -81,7 +79,12 @@ bool NERTagger::init( const Configuration& conf ){
     *Log(nerLog) << "Unable to find settings for NER" << endl;
     return false;
   }
-  string settings = val;
+  string settings;
+  if ( val[0] == '/' ) // an absolute path
+    settings = val;
+  else
+    settings =  configuration.configDir() + val;
+
   val = conf.lookUp( "version", "NER" );
   if ( val.empty() ){
     version = "1.0";
@@ -95,9 +98,9 @@ bool NERTagger::init( const Configuration& conf ){
   else
     tagset = val;
 
-  string init = "-s " + configuration.configDir() + settings + " -vcf";
+  string init = "-s " + settings + " -vcf";
   tagger = new MbtAPI( init, *nerLog );
-  return tagger != 0;
+  return tagger->isInit();
 }
 
 static void addEntity( EntitiesLayer *entities, 
@@ -126,13 +129,15 @@ static void addEntity( EntitiesLayer *entities,
   }
 }
 
-void NERTagger::addNERTags( Sentence *sent, 
-			    const vector<Word*>& words,
+void NERTagger::addNERTags( const vector<Word*>& words,
 			    const vector<string>& tags,
 			    const vector<double>& confs ){
+  if ( words.empty() )
+    return;
   EntitiesLayer *el = 0;
 #pragma omp critical(foliaupdate)
   {
+    Sentence *sent = words[0]->sentence();
     try {
       el = sent->annotation<EntitiesLayer>();
     }
@@ -152,7 +157,7 @@ void NERTagger::addNERTags( Sentence *sent,
       if ( !stack.empty() ){
 	if (debug) {
 	  *Log(nerLog) << "O spit out " << curNER << endl;
-	  using folia::operator<<;
+	  using TiCC::operator<<;
 	  *Log(nerLog) << "spit out " << stack << endl;	
 	}
 	addEntity( el, tagset, stack, dstack, curNER );
@@ -176,7 +181,7 @@ void NERTagger::addNERTags( Sentence *sent,
       if ( !stack.empty() ){
 	if ( debug ){
 	  *Log(nerLog) << "B spit out " << curNER << endl;
-	  using folia::operator<<;
+	  using TiCC::operator<<;
 	  *Log(nerLog) << "spit out " << stack << endl;	
 	}
 	addEntity( el, tagset, stack, dstack, curNER );
@@ -191,7 +196,7 @@ void NERTagger::addNERTags( Sentence *sent,
   if ( !stack.empty() ){
     if ( debug ){
       *Log(nerLog) << "END spit out " << curNER << endl;
-      using folia::operator<<;
+      using TiCC::operator<<;
       *Log(nerLog) << "spit out " << stack << endl;	
     }
     addEntity( el, tagset, stack, dstack, curNER );
@@ -202,15 +207,10 @@ void NERTagger::addDeclaration( Document& doc ) const {
   doc.declare( AnnotationType::ENTITY, 
 	       tagset,
 	       "annotator='frog-ner-" + version
-	       + "', annotatortype='auto'");
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
 
-void NERTagger::Classify( Sentence *sent ){
-  vector<Word*> swords;
-#pragma omp critical(foliaupdate)
-  {
-    swords = sent->words();
-  }
+void NERTagger::Classify( const vector<Word *>& swords ){
   if ( !swords.empty() ) {
     vector<string> words;
     string sentence; // the tagger needs the whole sentence
@@ -240,7 +240,7 @@ void NERTagger::Classify( Sentence *sent ){
       tags.push_back( tagv[i].assignedTag() );
       conf.push_back( tagv[i].confidence() );
     }
-    addNERTags( sent, swords, tags, conf );
+    addNERTags( swords, tags, conf );
   }
 }
 
