@@ -1,8 +1,8 @@
 /*
-  $Id: ucto_tokenizer_mod.cxx 14267 2012-02-15 13:05:47Z sloot $
+  $Id: ucto_tokenizer_mod.cxx 15833 2013-03-28 12:10:55Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/ucto_tokenizer_mod.cxx $
 
-  Copyright (c) 2006 - 2012
+  Copyright (c) 2006 - 2013
   Tilburg University
 
   A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
@@ -32,7 +32,7 @@
 #include <string>
 #include "timbl/TimblAPI.h"
 #include "frog/Frog.h"
-#include "frog/Configuration.h"
+#include "ticcutils/Configuration.h"
 #include "frog/ucto_tokenizer_mod.h"
 
 using namespace std;
@@ -48,7 +48,6 @@ bool UctoTokenizer::init( const Configuration& conf, const string & docid, bool
     throw runtime_error( "ucto tokenizer is already initalized" );
   tokenizer = new Tokenizer::TokenizerClass();
   tokenizer->setErrorLog( uctoLog );
-  *Log(uctoLog) << endl;
   string debug = conf.lookUp( "debug", "tokenizer" );
   if ( debug.empty() ){
     debug = conf.lookUp( "debug" );
@@ -57,7 +56,7 @@ bool UctoTokenizer::init( const Configuration& conf, const string & docid, bool
     tokenizer->setDebug( tpDebug );
   }
   else
-    tokenizer->setDebug( Timbl::stringTo<int>(debug) );
+    tokenizer->setDebug( TiCC::stringTo<int>(debug) );
   if ( pass ){
     // when passthru, we don't further initialize the tokenizer
     // it wil run in minimal mode then.
@@ -82,6 +81,15 @@ bool UctoTokenizer::init( const Configuration& conf, const string & docid, bool
   return true;
 }
 
+void UctoTokenizer::setUttMarker( const string& u ) {
+  if ( tokenizer ){
+    if ( !u.empty() )
+      tokenizer->setEosMarker( u ); 
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initalized" );
+};
+
 void UctoTokenizer::setSentencePerLineInput( bool b ) {
   if ( tokenizer )
     tokenizer->setSentencePerLineInput( b ); 
@@ -89,6 +97,13 @@ void UctoTokenizer::setSentencePerLineInput( bool b ) {
     throw runtime_error( "ucto tokenizer not initalized" );
 };
 
+void UctoTokenizer::setQuoteDetection( bool b ) {
+  if ( tokenizer )
+    tokenizer->setQuoteDetection( b ); 
+  else
+    throw runtime_error( "ucto tokenizer not initalized" );
+};
+
 void UctoTokenizer::setInputEncoding( const std::string & enc ){
   if ( tokenizer ){
     if ( !enc.empty() )
@@ -98,6 +113,15 @@ void UctoTokenizer::setInputEncoding( const std::string & enc ){
     throw runtime_error( "ucto tokenizer not initalized" );
 }
 
+void UctoTokenizer::setTextClass( const std::string& cls ){
+  if ( tokenizer ){
+    if ( !cls.empty() )
+      tokenizer->setTextClass( cls );
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initalized" );
+}
+
 void UctoTokenizer::setInputXml( bool b ){
   if ( tokenizer ){
     tokenizer->setXMLInput( b );
@@ -106,6 +130,21 @@ void UctoTokenizer::setInputXml( bool b ){
     throw runtime_error( "ucto tokenizer not initalized" );
 }
 
+vector<string> UctoTokenizer::tokenize( const string& line ){
+  if ( tokenizer ){
+    if ( tokenizer->getPassThru() ){
+      bool bos = true;
+      tokenizer->passthruLine( line, bos );
+    }
+    else {
+      tokenizer->tokenizeLine( line );
+    }
+    return tokenizer->getSentences();
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initalized" );
+}
+
 folia::Document UctoTokenizer::tokenize( istream& is ){
   if ( tokenizer )
     return tokenizer->tokenize( is );
